# Generated by Django 4.0.3 on 2022-09-02 03:18

from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        # changed dependency to last wagtail3 migration
        ("wagtailcore", "0069_log_entry_jsonfield"),
    ]

    operations = [
        migrations.CreateModel(
            name="SampleModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "content",
                    wagtail.fields.StreamField(
                        [
                            ("char1", wagtail.blocks.CharBlock()),
                            ("char2", wagtail.blocks.CharBlock()),
                            (
                                "simplestruct",
                                wagtail.blocks.StructBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        ("char2", wagtail.blocks.CharBlock()),
                                    ]
                                ),
                            ),
                            (
                                "simplestream",
                                wagtail.blocks.StreamBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        ("char2", wagtail.blocks.CharBlock()),
                                    ]
                                ),
                            ),
                            (
                                "simplelist",
                                wagtail.blocks.ListBlock(wagtail.blocks.CharBlock()),
                            ),
                            (
                                "nestedstruct",
                                wagtail.blocks.StructBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        (
                                            "stream1",
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "struct1",
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "list1",
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.CharBlock()
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                "nestedstream",
                                wagtail.blocks.StreamBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        (
                                            "stream1",
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "struct1",
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "list1",
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.CharBlock()
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                "nestedlist_struct",
                                wagtail.blocks.ListBlock(
                                    wagtail.blocks.StructBlock(
                                        [
                                            ("char1", wagtail.blocks.CharBlock()),
                                            ("char2", wagtail.blocks.CharBlock()),
                                        ]
                                    )
                                ),
                            ),
                            (
                                "nestedlist_stream",
                                wagtail.blocks.ListBlock(
                                    wagtail.blocks.StreamBlock(
                                        [
                                            ("char1", wagtail.blocks.CharBlock()),
                                            ("char2", wagtail.blocks.CharBlock()),
                                        ]
                                    )
                                ),
                            ),
                        ],
                        use_json_field=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="SamplePage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "content",
                    wagtail.fields.StreamField(
                        [
                            ("char1", wagtail.blocks.CharBlock()),
                            ("char2", wagtail.blocks.CharBlock()),
                            (
                                "simplestruct",
                                wagtail.blocks.StructBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        ("char2", wagtail.blocks.CharBlock()),
                                    ]
                                ),
                            ),
                            (
                                "simplestream",
                                wagtail.blocks.StreamBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        ("char2", wagtail.blocks.CharBlock()),
                                    ]
                                ),
                            ),
                            (
                                "simplelist",
                                wagtail.blocks.ListBlock(wagtail.blocks.CharBlock()),
                            ),
                            (
                                "nestedstruct",
                                wagtail.blocks.StructBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        (
                                            "stream1",
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "struct1",
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "list1",
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.CharBlock()
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                "nestedstream",
                                wagtail.blocks.StreamBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        (
                                            "stream1",
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "struct1",
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "list1",
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.CharBlock()
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                "nestedlist_struct",
                                wagtail.blocks.ListBlock(
                                    wagtail.blocks.StructBlock(
                                        [
                                            ("char1", wagtail.blocks.CharBlock()),
                                            ("char2", wagtail.blocks.CharBlock()),
                                        ]
                                    )
                                ),
                            ),
                            (
                                "nestedlist_stream",
                                wagtail.blocks.ListBlock(
                                    wagtail.blocks.StreamBlock(
                                        [
                                            ("char1", wagtail.blocks.CharBlock()),
                                            ("char2", wagtail.blocks.CharBlock()),
                                        ]
                                    )
                                ),
                            ),
                        ],
                        use_json_field=True,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
    ]



from django.contrib.admin.utils import quote
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.text import capfirst
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy
from django.views.generic import FormView

from wagtail.admin.forms.pages import ParentChooserForm
from wagtail.admin.views.generic.base import WagtailAdminTemplateMixin
from wagtail.models import Page
from wagtail.permissions import page_permission_policy


class ChooseParentView(WagtailAdminTemplateMixin, FormView):
    template_name = "wagtailadmin/pages/choose_parent.html"
    model = Page
    index_url_name = None
    page_title = gettext_lazy("Choose parent")

    def get_valid_parent_pages(self, user):
        """
        Identifies possible parent pages for the current user by first looking
        at allowed_parent_page_models() on self.model to limit options to the
        correct type of page, then checking permissions on those individual
        pages to make sure we have permission to add a subpage to it.
        """
        # Get queryset of pages where this page type can be added
        allowed_parent_page_content_types = list(
            ContentType.objects.get_for_models(
                *self.model.allowed_parent_page_models()
            ).values()
        )
        allowed_parent_pages = Page.objects.filter(
            content_type__in=allowed_parent_page_content_types
        )

        # Get queryset of pages where the user has permission to add subpages
        if user.is_superuser:
            pages_where_user_can_add = Page.objects.all()
        else:
            pages_where_user_can_add = Page.objects.none()

            perms = {
                perm
                for perm in page_permission_policy.get_cached_permissions_for_user(user)
                if perm.permission.codename == "add_page"
            }

            for perm in perms:
                # user has add permission on any subpage of perm.page
                # (including perm.page itself)
                pages_where_user_can_add |= Page.objects.descendant_of(
                    perm.page, inclusive=True
                )

        # Combine them
        return allowed_parent_pages & pages_where_user_can_add

    def get(self, request, *args, **kwargs):
        parents = self.get_valid_parent_pages(request.user)

        # Only fetch the IDs for the first two parents to check if there's only
        # one valid parent, and if so, redirect to the add page view with that
        # parent pre-selected
        parent_ids = parents.values_list("pk", flat=True)[:2]
        if len(parent_ids) == 1:
            parent_id = quote(parent_ids[0])
            model_opts = self.model._meta
            return redirect(
                "wagtailadmin_pages:add",
                model_opts.app_label,
                model_opts.model_name,
                parent_id,
            )

        # The page can be added in multiple places, so proceed with rendering
        # the view's form so that the parent can be specified
        return super().get(request, *args, **kwargs)

    def get_form(self):
        if self.request.method == "POST":
            return ParentChooserForm(self.model, self.request.user, self.request.POST)
        return ParentChooserForm(self.model, self.request.user)

    def get_index_url(self):
        if self.index_url_name:
            return reverse(self.index_url_name)

    def get_breadcrumbs_items(self):
        items = []
        index_url = self.get_index_url()
        if index_url:
            items.append(
                {
                    "url": index_url,
                    "label": capfirst(self.model._meta.verbose_name_plural),
                }
            )
        items.append(
            {
                "url": "",
                "label": self.get_page_title(),
                "sublabel": self.get_page_subtitle(),
            }
        )

        return self.breadcrumbs_items + items

    def get_page_subtitle(self):
        return self.model.get_verbose_name()

    @cached_property
    def submit_button_label(self):
        return _("Create a new %(model_name)s") % {
            "model_name": self.model._meta.verbose_name,
        }

    def form_valid(self, form):
        model_opts = self.model._meta
        parent_id = quote(form.cleaned_data["parent_page"].pk)
        return redirect(
            "wagtailadmin_pages:add",
            model_opts.app_label,
            model_opts.model_name,
            parent_id,
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["media"] = context["form"].media
        context["submit_button_label"] = self.submit_button_label
        return context



from django.urls import path

from wagtail.admin.views.pages.choose_parent import ChooseParentView
from wagtail.admin.views.pages.listing import IndexView
from wagtail.models import Page

from .base import ViewSet


class PageListingViewSet(ViewSet):
    """
    A viewset to present a flat listing of all pages of a specific type.
    All attributes and methods from :class:`~wagtail.admin.viewsets.base.ViewSet`
    are available.
    For more information on how to use this class, see :ref:`custom_page_listings`.
    """

    #: The view class to use for the index view; must be a subclass of ``wagtail.admin.views.pages.listing.IndexView``.
    index_view_class = IndexView
    #: The view class to use for choosing the parent page when creating a new page of this page type.
    choose_parent_view_class = ChooseParentView
    #: Required; the page model class that this viewset will work with.
    model = Page
    #: A list of ``wagtail.admin.ui.tables.Column`` instances for the columns in the listing.
    columns = IndexView.columns
    #: A subclass of ``wagtail.admin.filters.WagtailFilterSet``, which is a
    #: subclass of `django_filters.FilterSet <https://django-filter.readthedocs.io/en/stable/ref/filterset.html>`_.
    #: This will be passed to the ``filterset_class`` attribute of the index view.
    filterset_class = IndexView.filterset_class

    def get_common_view_kwargs(self, **kwargs):
        return super().get_common_view_kwargs(
            **{
                "header_icon": self.icon,
                "model": self.model,
                "index_url_name": self.get_url_name("index"),
                "add_url_name": self.get_url_name("choose_parent"),
                **kwargs,
            }
        )

    def get_index_view_kwargs(self, **kwargs):
        return {
            "index_results_url_name": self.get_url_name("index_results"),
            "columns": self.columns,
            "filterset_class": self.filterset_class,
            **kwargs,
        }

    def get_choose_parent_view_kwargs(self, **kwargs):
        return kwargs

    @property
    def index_view(self):
        return self.construct_view(
            self.index_view_class, **self.get_index_view_kwargs()
        )

    @property
    def index_results_view(self):
        return self.construct_view(
            self.index_view_class, **self.get_index_view_kwargs(), results_only=True
        )

    @property
    def choose_parent_view(self):
        return self.construct_view(
            self.choose_parent_view_class, **self.get_choose_parent_view_kwargs()
        )

    def get_urlpatterns(self):
        return [
            path("", self.index_view, name="index"),
            path("results/", self.index_results_view, name="index_results"),
            path("choose_parent/", self.choose_parent_view, name="choose_parent"),
        ]



# Generated by Django 5.0.1 on 2024-03-15 21:20

import django.db.models.manager
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("tests", "0034_custompermissionmodel"),
    ]

    operations = [
        migrations.CreateModel(
            name="ModelWithCustomManager",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            managers=[
                ("instances", django.db.models.manager.Manager()),
            ],
        ),
    ]



import hashlib
import os
import pickle
import tempfile
import unittest
from io import BytesIO
from pathlib import Path

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import SimpleTestCase, TestCase, override_settings
from django.utils.text import slugify
from django.utils.translation import _trans
from django.utils.translation import gettext_lazy as _

from wagtail.coreutils import (
    InvokeViaAttributeShortcut,
    accepts_kwarg,
    camelcase_to_underscore,
    cautious_slugify,
    find_available_slug,
    get_content_languages,
    get_content_type_label,
    get_dummy_request,
    get_supported_content_language_variant,
    multigetattr,
    safe_snake_case,
    string_to_ascii,
)
from wagtail.models import Page, Site
from wagtail.utils.file import hash_filelike
from wagtail.utils.templates import template_is_overridden
from wagtail.utils.utils import deep_update, flatten_choices
from wagtail.utils.version import get_main_version


class TestCamelCaseToUnderscore(TestCase):
    def test_camelcase_to_underscore(self):
        test_cases = [
            ("HelloWorld", "hello_world"),
            ("longValueWithVarious subStrings", "long_value_with_various sub_strings"),
        ]

        for original, expected_result in test_cases:
            self.assertEqual(camelcase_to_underscore(original), expected_result)


class TestStringToAscii(TestCase):
    def test_string_to_ascii(self):
        test_cases = [
            ("30 \U0001d5c4\U0001d5c6/\U0001d5c1", "30 km/h"),
            ("\u5317\u4eb0", "BeiJing"),
            ("ぁ あ ぃ い ぅ う ぇ", "a a i i u u e"),
            (
                "Ա Բ Գ Դ Ե Զ Է Ը Թ Ժ Ի Լ Խ Ծ Կ Հ Ձ Ղ Ճ Մ Յ Ն",
                "A B G D E Z E Y T' Zh I L Kh Ts K H Dz Gh Ch M Y N",
            ),
            ("Спорт!", "Sport!"),
            ("Straßenbahn", "Strassenbahn"),
            ("Hello world", "Hello world"),
            ("Ā ā Ă ă Ą ą Ć ć Ĉ ĉ Ċ ċ Č č Ď ď Đ", "A a A a A a C c C c C c C c D d D"),
            ("〔山脈〕", "[ShanMai]"),
        ]

        for original, expected_result in test_cases:
            self.assertEqual(string_to_ascii(original), expected_result)


class TestCautiousSlugify(TestCase):
    def test_behaves_same_as_slugify_for_latin_chars(self):
        test_cases = [
            ("", ""),
            ("???", ""),
            ("Hello world", "hello-world"),
            ("Hello_world", "hello_world"),
            ("Hellö wörld", "hello-world"),
            ("Hello   world", "hello-world"),
            ("   Hello world   ", "hello-world"),
            ("Hello, world!", "hello-world"),
            ("Hello*world", "helloworld"),
            ("Hello☃world", "helloworld"),
        ]

        for original, expected_result in test_cases:
            self.assertEqual(slugify(original), expected_result)
            self.assertEqual(cautious_slugify(original), expected_result)

    def test_escapes_non_latin_chars(self):
        test_cases = [
            ("Straßenbahn", "straxdfenbahn"),
            ("Спорт!", "u0421u043fu043eu0440u0442"),
            ("〔山脈〕", "u5c71u8108"),
        ]

        for original, expected_result in test_cases:
            self.assertEqual(cautious_slugify(original), expected_result)


class TestSafeSnakeCase(TestCase):
    def test_strings_with_latin_chars(self):
        test_cases = [
            ("", ""),
            ("???", ""),
            ("using-Hyphen", "using_hyphen"),
            ("en–⁠dash", "endash"),  # unicode non-letter characters stripped
            ("  em—dash ", "emdash"),  # unicode non-letter characters stripped
            (
                "horizontal―BAR",
                "horizontalbar",
            ),  # unicode non-letter characters stripped
            ("Hello world", "hello_world"),
            ("Hello_world", "hello_world"),
            ("Hellö wörld", "hello_world"),
            ("Hello   world", "hello_world"),
            ("   Hello world   ", "hello_world"),
            ("Hello, world!", "hello_world"),
            ("Hello*world", "helloworld"),
            (
                "Screenshot_2020-05-29 Screenshot(1).png",
                "screenshot_2020_05_29_screenshot1png",
            ),
        ]

        for original, expected_result in test_cases:
            self.assertEqual(safe_snake_case(original), expected_result)

    def test_strings_with__non_latin_chars(self):
        test_cases = [
            ("Straßenbahn Straßenbahn", "straxdfenbahn_straxdfenbahn"),
            ("Сп орт!", "u0421u043f_u043eu0440u0442"),
        ]

        for original, expected_result in test_cases:
            self.assertEqual(safe_snake_case(original), expected_result)


class TestAcceptsKwarg(TestCase):
    def test_accepts_kwarg(self):
        def func_without_banana(apple, orange=42):
            pass

        def func_with_banana(apple, banana=42):
            pass

        def func_with_kwargs(apple, **kwargs):
            pass

        self.assertFalse(accepts_kwarg(func_without_banana, "banana"))
        self.assertTrue(accepts_kwarg(func_with_banana, "banana"))
        self.assertTrue(accepts_kwarg(func_with_kwargs, "banana"))


class TestTargetClass:
    """
    Used in TestInvokeViaAttributeShortcut (below)
    """

    def __init__(self):
        self.target_method_called_with = []

    def target_method(self, arg):
        self.target_method_called_with.append(arg)


class TestInvokeViaAttributeShortcut(SimpleTestCase):
    def setUp(self):
        self.target_object = TestTargetClass()
        self.test_object = InvokeViaAttributeShortcut(
            self.target_object, "target_method"
        )

    def test_basic(self):
        for value in ("foo", "bar", "baz"):
            # Use the shortcut to call the underlying method
            getattr(self.test_object, value)
            # Confirm that the underlying method was called
            self.assertIn(value, self.target_object.target_method_called_with)

    def test_pickleability(self):
        try:
            pickled = pickle.dumps(self.test_object, -1)
        except Exception as e:  # noqa: BLE001
            raise AssertionError(
                "An error occurred when attempting to pickle %r: %s"
                % (self.test_object, e)
            )
        try:
            self.test_object = pickle.loads(pickled)
        except Exception as e:  # noqa: BLE001
            raise AssertionError(
                "An error occurred when attempting to unpickle %r: %s"
                % (self.test_object, e)
            )

        # Confirm unpickled object works the same
        self.target_object = self.test_object.obj
        self.test_basic()


class TestFindAvailableSlug(TestCase):
    def setUp(self):
        self.root_page = Page.objects.get(depth=1)
        self.home_page = Page.objects.get(depth=2)

        self.second_home_page = self.root_page.add_child(
            instance=Page(title="Second homepage", slug="home-1")
        )

    def test_find_available_slug(self):
        with self.assertNumQueries(1):
            slug = find_available_slug(self.root_page, "unique-slug")

        self.assertEqual(slug, "unique-slug")

    def test_find_available_slug_already_used(self):
        # Even though the first two slugs are already used, this still requires only one query to find a unique one
        with self.assertNumQueries(1):
            slug = find_available_slug(self.root_page, "home")

        self.assertEqual(slug, "home-2")

    def test_find_available_slug_ignore_page_id(self):
        with self.assertNumQueries(1):
            slug = find_available_slug(
                self.root_page, "home", ignore_page_id=self.second_home_page.id
            )

        self.assertEqual(slug, "home-1")


@override_settings(
    USE_I18N=True,
    WAGTAIL_I18N_ENABLED=True,
    LANGUAGES=[
        ("en", "English"),
        ("de", "German"),
        ("de-at", "Austrian German"),
        ("pt-br", "Portuguese (Brazil)"),
    ],
    WAGTAIL_CONTENT_LANGUAGES=[
        ("en", "English"),
        ("de", "German"),
        ("de-at", "Austrian German"),
        ("pt-br", "Portuguese (Brazil)"),
    ],
)
class TestGetContentLanguages(TestCase):
    def test_get_content_languages(self):
        self.assertEqual(
            get_content_languages(),
            {
                "de": "German",
                "de-at": "Austrian German",
                "en": "English",
                "pt-br": "Portuguese (Brazil)",
            },
        )

    @override_settings(
        WAGTAIL_CONTENT_LANGUAGES=[
            ("en", "English"),
            ("de", "German"),
        ],
    )
    def test_can_be_different_to_django_languages(self):
        self.assertEqual(
            get_content_languages(),
            {
                "de": "German",
                "en": "English",
            },
        )

    @override_settings(
        WAGTAIL_CONTENT_LANGUAGES=[
            ("en", _("English")),
            ("de", _("German")),
        ],
    )
    def test_can_be_a_translation_proxy(self):
        self.assertEqual(
            get_content_languages(),
            {
                "de": "German",
                "en": "English",
            },
        )

    @override_settings(
        WAGTAIL_CONTENT_LANGUAGES=[
            ("en", "English"),
            ("de", "German"),
            ("zh", "Chinese"),
        ],
    )
    def test_must_be_subset_of_django_languages(self):
        with self.assertRaises(ImproperlyConfigured) as e:
            get_content_languages()

        self.assertEqual(
            e.exception.args,
            (
                "The language zh is specified in WAGTAIL_CONTENT_LANGUAGES but not LANGUAGES. WAGTAIL_CONTENT_LANGUAGES must be a subset of LANGUAGES.",
            ),
        )


def TestGetContentTypeLabel(TestCase):
    def test_none(self):
        self.assertEqual(get_content_type_label(None), "Unknown content type")

    def test_valid_content_type(self):
        page_content_type = ContentType.objects.get_for_model(Page)
        self.assertEqual(get_content_type_label(page_content_type), "Page")

    def test_stale_content_type(self):
        stale_content_type = ContentType.objects.create(
            app_label="fake_app", model="deleted model"
        )
        self.assertEqual(get_content_type_label(stale_content_type), "Deleted model")


@override_settings(
    USE_I18N=True,
    WAGTAIL_I18N_ENABLED=True,
    LANGUAGES=[
        ("en", "English"),
        ("de", "German"),
        ("de-at", "Austrian German"),
        ("pt-br", "Portuguese (Brazil)"),
    ],
    WAGTAIL_CONTENT_LANGUAGES=[
        ("en", "English"),
        ("de", "German"),
        ("de-at", "Austrian German"),
        ("pt-br", "Portuguese (Brazil)"),
    ],
)
class TestGetSupportedContentLanguageVariant(TestCase):
    # From: https://github.com/django/django/blob/9e57b1efb5205bd94462e9de35254ec5ea6eb04e/tests/i18n/tests.py#L1481
    def test_get_supported_content_language_variant(self):
        g = get_supported_content_language_variant
        self.assertEqual(g("en"), "en")
        self.assertEqual(g("en-gb"), "en")
        self.assertEqual(g("de"), "de")
        self.assertEqual(g("de-at"), "de-at")
        self.assertEqual(g("de-ch"), "de")
        self.assertEqual(g("pt-br"), "pt-br")
        self.assertEqual(g("pt"), "pt-br")
        self.assertEqual(g("pt-pt"), "pt-br")
        with self.assertRaises(LookupError):
            g("pt", strict=True)
        with self.assertRaises(LookupError):
            g("pt-pt", strict=True)
        with self.assertRaises(LookupError):
            g("xyz")
        with self.assertRaises(LookupError):
            g("xy-zz")

    @override_settings(
        WAGTAIL_CONTENT_LANGUAGES=[
            ("en", "English"),
            ("de", "German"),
        ]
    )
    def test_uses_wagtail_content_languages(self):
        # be sure it's not using Django's LANGUAGES
        g = get_supported_content_language_variant
        self.assertEqual(g("en"), "en")
        self.assertEqual(g("en-gb"), "en")
        self.assertEqual(g("de"), "de")
        self.assertEqual(g("de-at"), "de")
        self.assertEqual(g("de-ch"), "de")
        with self.assertRaises(LookupError):
            g("pt-br")
        with self.assertRaises(LookupError):
            g("pt")
        with self.assertRaises(LookupError):
            g("pt-pt")
        with self.assertRaises(LookupError):
            g("xyz")
        with self.assertRaises(LookupError):
            g("xy-zz")


@override_settings(
    USE_I18N=False,
    WAGTAIL_I18N_ENABLED=False,
    WAGTAIL_CONTENT_LANGUAGES=None,
    LANGUAGE_CODE="en-us",
)
class TestGetSupportedContentLanguageVariantWithI18nFalse(TestCase):
    def setUp(self):
        # Need to forcibly clear the django.utils.translation._trans object when overriding
        # USE_I18N:
        # https://github.com/django/django/blob/3.1/django/utils/translation/__init__.py#L46-L48
        _trans.__dict__.clear()

    def tearDown(self):
        _trans.__dict__.clear()

    def test_lookup_language_with_i18n_false(self):
        # Make sure we can handle the 'null' USE_I18N=False implementation of
        # get_supported_language_variant returning 'en-us' rather than 'en',
        # despite 'en-us' not being in LANGUAGES.
        # https://github.com/wagtail/wagtail/issues/6539

        self.assertEqual(get_supported_content_language_variant("en-us"), "en")

    @override_settings(LANGUAGE_CODE="zz")
    def test_language_code_not_in_languages(self):
        # Ensure we can handle a LANGUAGE_CODE setting that isn't defined in LANGUAGES -
        # in this case get_content_languages has to cope with not being able to retrieve
        # a display name for the language
        self.assertEqual(get_supported_content_language_variant("zz"), "zz")
        self.assertEqual(get_supported_content_language_variant("zz-gb"), "zz")


class TestMultigetattr(TestCase):
    def setUp(self):
        class Thing:
            colour = "green"
            limbs = {"arms": 2, "legs": 3}

            def __init__(self):
                self.poke_was_called = False

            def speak(self):
                return "raaargh"

            def feed(self, food):
                return "gobble"

            def poke(self):
                self.poke_was_called = True
                raise Exception("don't do that")

            poke.alters_data = True

        self.thing = Thing()

    def test_multigetattr(self):
        self.assertEqual(multigetattr(self.thing, "colour"), "green")
        self.assertEqual(multigetattr(self, "thing.colour"), "green")
        self.assertEqual(multigetattr(self.thing, "limbs.arms"), 2)
        self.assertEqual(multigetattr(self.thing, "speak"), "raaargh")
        self.assertEqual(multigetattr(self, "thing.speak.0"), "r")

        with self.assertRaises(AttributeError):
            multigetattr(self.thing, "name")

        with self.assertRaises(AttributeError):
            multigetattr(self.thing, "limbs.antennae")

        with self.assertRaises(AttributeError):
            multigetattr(self.thing, "speak.999")

        with self.assertRaises(TypeError):
            multigetattr(self.thing, "feed")

        with self.assertRaises(SuspiciousOperation):
            multigetattr(self.thing, "poke")
        self.assertFalse(self.thing.poke_was_called)


class TestGetDummyRequest(TestCase):
    def test_standard_port(self):
        site = Site.objects.first()
        site.hostname = "other.example.com"
        site.port = 80
        site.save()

        request = get_dummy_request(site=site)
        self.assertEqual(request.get_host(), "other.example.com")

    def test_non_standard_port(self):
        site = Site.objects.first()
        site.hostname = "other.example.com"
        site.port = 8888
        site.save()

        request = get_dummy_request(site=site)
        self.assertEqual(request.get_host(), "other.example.com:8888")

    def test_server_name_for_wildcard_allowed_hosts(self):
        # Django's test runner adds "testserver" at the end of ALLOWED_HOSTS.
        with self.settings(ALLOWED_HOSTS=["*", "testserver"]):
            request = get_dummy_request()
            self.assertEqual(request.get_host(), "example.com")


class TestDeepUpdate(TestCase):
    def test_deep_update(self):
        val = {
            "captain": "picard",
            "beverage": {
                "type": "coffee",
                "temperature": "hot",
            },
        }

        deep_update(
            val,
            {
                "beverage": {
                    "type": "tea",
                    "variant": "earl grey",
                },
                "starship": "enterprise",
            },
        )

        self.assertEqual(
            val,
            {
                "captain": "picard",
                "beverage": {
                    "type": "tea",
                    "variant": "earl grey",
                    "temperature": "hot",
                },
                "starship": "enterprise",
            },
        )


class HashFileLikeTestCase(SimpleTestCase):
    test_file = Path.cwd() / "LICENSE"

    def test_hashes_io(self):
        self.assertEqual(
            hash_filelike(BytesIO(b"test")), "a94a8fe5ccb19ba61c4c0873d391e987982fbbd3"
        )

    def test_hashes_file(self):
        with self.test_file.open(mode="rb") as f:
            self.assertEqual(
                hash_filelike(f), "9e58400061ca660ef7b5c94338a5205627c77eda"
            )

    def test_hashes_file_bytes(self):
        with self.test_file.open(mode="rb") as f:
            self.assertEqual(
                hash_filelike(f), "9e58400061ca660ef7b5c94338a5205627c77eda"
            )

    def test_hashes_django_uploaded_file(self):
        """
        Check Django's file shims can be hashed as-is.
        `SimpleUploadedFile` inherits the base `UploadedFile`, but is easiest to test against
        """
        self.assertEqual(
            hash_filelike(SimpleUploadedFile("example.txt", b"test")),
            "a94a8fe5ccb19ba61c4c0873d391e987982fbbd3",
        )

    @unittest.skipIf(
        hasattr(hashlib, "file_digest"),
        reason="`file_digest` doesn't support this interface",
    )
    def test_hashes_large_file(self):
        class FakeLargeFile:
            """
            A class that pretends to be a huge file (~1.3GB)
            """

            def __init__(self):
                self.iterations = 5000

            def read(self, bytes):
                self.iterations -= 1
                if not self.iterations:
                    return b""

                return b"A" * bytes

        self.assertEqual(
            hash_filelike(FakeLargeFile()),
            "bd36f0c5a02cd6e9e34202ea3ff8db07b533e025",
        )


class TestTemplateIsOverridden(SimpleTestCase):
    def setUp(self):
        template_is_overridden.cache_clear()

    def test_template_is_overridden_false(self):
        self.assertIs(
            template_is_overridden(
                "wagtailcore/shared/block_preview.html",
                "templates",
            ),
            False,
        )

    def test_template_is_overridden_true(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            path = os.path.join(temp_dir, "wagtailcore/shared")
            os.makedirs(path, exist_ok=True)
            with open(os.path.join(path, "block_preview.html"), "w") as f:
                f.write("Custom file")

            with self.settings(
                TEMPLATES=[
                    {**settings.TEMPLATES[0], "DIRS": [temp_dir], "NAME": "tmp"},
                    *settings.TEMPLATES,
                ]
            ):
                self.assertIs(
                    template_is_overridden(
                        "wagtailcore/shared/block_preview.html",
                        "templates",
                    ),
                    True,
                )


class TestVersion(SimpleTestCase):
    def test_get_main_version(self):
        cases = [
            ((6, 2, 0, "final", 0), False, "6.2"),
            ((6, 2, 1, "final", 0), False, "6.2"),
            ((6, 2, 0, "final", 0), True, "6.2"),
            ((6, 2, 1, "final", 0), True, "6.2.1"),
        ]
        for version, include_patch, expected in cases:
            with self.subTest(version=version, include_patch=include_patch):
                self.assertEqual(get_main_version(version, include_patch), expected)


class TestFlattenChoices(SimpleTestCase):
    def test_tuple_choices(self):
        choices = [(1, "1st"), (2, "2nd")]
        self.assertEqual(flatten_choices(choices), {"1": "1st", "2": "2nd"})

    def test_grouped_tuple_choices(self):
        choices = [("Group", [(1, "1st"), (2, "2nd")])]
        self.assertEqual(flatten_choices(choices), {"1": "1st", "2": "2nd"})

    def test_dictionary_choices(self):
        choices = {
            "Martial Arts": {"judo": "Judo", "karate": "Karate"},
            "Racket": {"badminton": "Badminton", "tennis": "Tennis"},
            "unknown": "Unknown",
        }
        self.assertEqual(
            flatten_choices(choices),
            {
                "judo": "Judo",
                "karate": "Karate",
                "badminton": "Badminton",
                "tennis": "Tennis",
                "unknown": "Unknown",
            },
        )



# Generated by Django 5.0.9 on 2024-10-09 16:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildocs", "0013_delete_uploadeddocument"),
    ]

    operations = [
        migrations.AlterField(
            model_name="document",
            name="file_size",
            field=models.PositiveBigIntegerField(editable=False, null=True),
        ),
    ]



from wagtail.users.views.users import UserViewSet as WagtailUserViewSet

from .forms import CustomUserCreationForm, CustomUserEditForm


class CustomUserViewSet(WagtailUserViewSet):
    icon = "custom-icon"

    def get_form_class(self, for_update=False):
        if for_update:
            return CustomUserEditForm
        return CustomUserCreationForm



from django.db import transaction
from django.db.models.signals import post_delete

from wagtail.documents import get_document_model
from wagtail.tasks import delete_file_from_storage_task


def post_delete_file_cleanup(instance, **kwargs):
    transaction.on_commit(
        lambda: delete_file_from_storage_task.enqueue(
            instance.file.storage.deconstruct(), instance.file.name
        )
    )


def register_signal_handlers():
    Document = get_document_model()
    post_delete.connect(post_delete_file_cleanup, sender=Document)



from django.template import Context, RequestContext, Template, engines
from django.test import TestCase
from django.test.utils import override_settings

from wagtail.coreutils import get_dummy_request
from wagtail.models import Site
from wagtail.test.utils import WagtailTestUtils

from .base import SiteSettingsTestMixin


@override_settings(ALLOWED_HOSTS=["testserver", "localhost", "other"])
class TemplateTestCase(SiteSettingsTestMixin, WagtailTestUtils, TestCase):
    def render(self, request, string, context=None, site=None):
        template = Template(string)
        context = RequestContext(request, context)
        return template.render(context)


class TestContextProcessor(TemplateTestCase):
    def test_accessing_setting(self):
        """Check that the context processor works"""
        request = self.get_request()
        self.assertEqual(
            self.render(request, "{{ settings.tests.TestSiteSetting.title }}"),
            self.default_settings.title,
        )

    def test_multi_site(self):
        """Check that the correct setting for the current site is returned"""
        request = self.get_request(site=self.default_site)
        self.assertEqual(
            self.render(request, "{{ settings.tests.TestSiteSetting.title }}"),
            self.default_settings.title,
        )

        request = self.get_request(site=self.other_site)
        self.assertEqual(
            self.render(request, "{{ settings.tests.TestSiteSetting.title }}"),
            self.other_settings.title,
        )

    def test_model_case_insensitive(self):
        """Model names should be case insensitive"""
        request = self.get_request()
        self.assertEqual(
            self.render(request, "{{ settings.tests.testsitesetting.title }}"),
            self.default_settings.title,
        )
        self.assertEqual(
            self.render(request, "{{ settings.tests.TESTSITESETTING.title }}"),
            self.default_settings.title,
        )
        self.assertEqual(
            self.render(request, "{{ settings.tests.TestSiteSetting.title }}"),
            self.default_settings.title,
        )
        self.assertEqual(
            self.render(request, "{{ settings.tests.tEstSiTesEttIng.title }}"),
            self.default_settings.title,
        )

    def test_models_cached(self):
        """Accessing a setting should only hit the DB once per request instance,
        even if using that request to rendering multiple times"""
        request = self.get_request()
        get_title = "{{ settings.tests.testsitesetting.title }}"

        # force site query beforehand
        Site.find_for_request(request)

        with self.assertNumQueries(1):
            for i in range(1, 4):
                with self.subTest(attempt=i):
                    self.assertEqual(
                        self.render(request, get_title * i),
                        self.default_settings.title * i,
                    )


class TestTemplateTag(TemplateTestCase):
    def test_no_context_processor(self):
        """
        Assert that not running the context processor means settings are not in
        the context, as expected.
        """
        template = Template("{{ settings.tests.TestSiteSetting.title }}")
        context = Context()
        self.assertEqual(template.render(context), "")

    def test_get_settings_request_context(self):
        """Check that the {% get_settings %} tag works"""
        request = self.get_request(site=self.other_site)
        context = Context({"request": request})

        # This should use the site in the request
        template = Template(
            "{% load wagtailsettings_tags %}"
            "{% get_settings %}"
            "{{ settings.tests.testsitesetting.title }}"
        )

        self.assertEqual(template.render(context), self.other_settings.title)

    def test_get_settings_request_context_use_default(self):
        """
        Check that the {% get_settings use_default_site=True %} option
        overrides a request in the context.
        """
        request = self.get_request(site=self.other_site)
        context = Context({"request": request})

        # This should use the default site, ignoring the site in the request
        template = Template(
            "{% load wagtailsettings_tags %}"
            "{% get_settings use_default_site=True %}"
            "{{ settings.tests.testsitesetting.title }}"
        )

        self.assertEqual(template.render(context), self.default_settings.title)

    def test_get_settings_use_default(self):
        """
        Check that the {% get_settings use_default_site=True %} option works
        """
        context = Context()

        # This should use the default site
        template = Template(
            "{% load wagtailsettings_tags %}"
            "{% get_settings use_default_site=True %}"
            "{{ settings.tests.testsitesetting.title }}"
        )

        self.assertEqual(template.render(context), self.default_settings.title)

    def test_get_settings_no_request_no_default(self):
        """
        Check that the {% get_settings %} throws an error if it can not find a
        site to work with
        """
        context = Context()

        # Without a request in the context, and without use_default_site, this
        # should bail with an error
        template = Template(
            "{% load wagtailsettings_tags %}"
            "{% get_settings %}"
            "{{ settings.tests.testsitesetting.title }}"
        )
        with self.assertRaises(RuntimeError):
            template.render(context)

    def test_get_settings_variable_assignment_request_context(self):
        """
        Check that assigning the setting to a context variable with
        {% get_settings as wagtail_settings %} works.
        """
        request = self.get_request(site=self.other_site)
        context = Context({"request": request})
        template = Template(
            "{% load wagtailsettings_tags %}"
            "{% get_settings as wagtail_settings %}"
            "{{ wagtail_settings.tests.testsitesetting.title }}"
        )

        self.assertEqual(template.render(context), self.other_settings.title)
        # Also check that the default 'settings' variable hasn't been set
        template = Template(
            "{% load wagtailsettings_tags %}"
            "{% get_settings as wagtail_settings %}"
            "{{ settings.tests.testsitesetting.title }}"
        )

        self.assertEqual(template.render(context), "")

    def test_get_settings_variable_assignment_use_default(self):
        """
        Check that assigning the setting to a context variable with
        {% get_settings use_default_site=True as wagtail_settings %} works.
        """
        context = Context()
        template = Template(
            "{% load wagtailsettings_tags %}"
            "{% get_settings use_default_site=True as wagtail_settings %}"
            "{{ wagtail_settings.tests.testsitesetting.title }}"
        )

        self.assertEqual(template.render(context), self.default_settings.title)


class TestSiteSettingsJinja(TemplateTestCase):
    def setUp(self):
        super().setUp()
        self.engine = engines["jinja2"]

    def render(self, string, context=None, request_context=True):
        if context is None:
            context = {}

        # Add a request to the template, to simulate a RequestContext
        if request_context:
            if "site" in context:
                site = context["site"]
            else:
                site = Site.objects.get(is_default_site=True)
            context["request"] = get_dummy_request(site=site)

        template = self.engine.from_string(string)
        return template.render(context)

    def test_accessing_setting(self):
        """Check that the context processor works"""
        self.assertEqual(
            self.render('{{ settings("tests.TestSiteSetting").title }}'),
            self.default_settings.title,
        )

    def test_multi_site(self):
        """Check that the correct setting for the current site is returned"""
        context = {"site": self.default_site}
        self.assertEqual(
            self.render(
                '{{ settings("tests.TestSiteSetting").title }}',
                context,
            ),
            self.default_settings.title,
        )

        context = {"site": self.other_site}
        self.assertEqual(
            self.render(
                '{{ settings("tests.TestSiteSetting").title }}',
                context,
            ),
            self.other_settings.title,
        )

    def test_model_case_insensitive(self):
        """Model names should be case insensitive"""
        self.assertEqual(
            self.render('{{ settings("tests.testsitesetting").title }}'),
            self.default_settings.title,
        )
        self.assertEqual(
            self.render('{{ settings("tests.TESTSITESETTING").title }}'),
            self.default_settings.title,
        )
        self.assertEqual(
            self.render('{{ settings("tests.TestSiteSetting").title }}'),
            self.default_settings.title,
        )
        self.assertEqual(
            self.render('{{ settings("tests.tEstSiTesEttIng").title }}'),
            self.default_settings.title,
        )

    def test_models_cached(self):
        """Accessing a setting should only hit the DB once per render"""
        get_title = '{{ settings("tests.testsitesetting").title }}'

        request = self.get_request()
        # run extra query before hand
        Site.find_for_request(request)

        for i in range(1, 4):
            with self.assertNumQueries(1):
                context = {"request": request}
                template = self.engine.from_string(get_title * i)
                self.assertEqual(
                    template.render(context), self.default_settings.title * i
                )

    def test_settings_use_default_site_override(self):
        """
        Check that {{ settings(use_default_site=True) }} overrides a site in
        the context.
        """
        request = self.get_request(site=self.other_site)
        context = {"request": request}

        # This should use the default site, ignoring the site in the request
        template = (
            '{{ settings("tests.testsitesetting", use_default_site=True).title }}'
        )

        self.assertEqual(self.render(template, context), self.default_settings.title)

    def test_settings_use_default_site(self):
        """
        Check that the {{ settings(use_default_site=True) }} option works with
        no site in the context
        """
        context = {}

        # This should use the default site
        template = (
            '{{ settings("tests.testsitesetting", use_default_site=True).title }}'
        )

        self.assertEqual(
            self.render(template, context, request_context=False),
            self.default_settings.title,
        )

    def test_settings_no_request_no_use_default(self):
        """
        Check that {{ settings }} throws an error if it can not find a
        site to work with
        """
        context = {}

        # Without a request in the context, and without use_default_site, this
        # should bail with an error
        template = '{{ settings("tests.testsitesetting").title }}'
        with self.assertRaises(RuntimeError):
            self.render(template, context, request_context=False)



from django.db import models
from taggit.managers import TaggableManager

from wagtail.search import index


class Author(index.Indexed, models.Model):
    name = models.CharField(max_length=255)
    date_of_birth = models.DateField(null=True)

    search_fields = [
        index.SearchField("name"),
        index.AutocompleteField("name"),
        index.FilterField("date_of_birth"),
    ]

    def __str__(self):
        return self.name


class Book(index.Indexed, models.Model):
    title = models.CharField(max_length=255)
    summary = models.TextField(blank=True)
    authors = models.ManyToManyField(Author, related_name="books")
    publication_date = models.DateField()
    number_of_pages = models.IntegerField()
    tags = TaggableManager()

    search_fields = [
        index.SearchField("title", boost=2.0),
        index.SearchField("summary"),
        index.AutocompleteField("title"),
        index.FilterField("title"),
        index.FilterField("authors"),
        index.RelatedFields("authors", Author.search_fields),
        index.FilterField("publication_date"),
        index.FilterField("number_of_pages"),
        index.RelatedFields(
            "tags",
            [
                index.SearchField("name"),
                index.FilterField("slug"),
            ],
        ),
        index.FilterField("tags"),
    ]

    @classmethod
    def get_indexed_objects(cls):
        indexed_objects = super().get_indexed_objects()

        # Don't index books using Book class that they have a more specific type
        if cls is Book:
            indexed_objects = indexed_objects.exclude(
                id__in=Novel.objects.values_list("book_ptr_id", flat=True)
            )

            indexed_objects = indexed_objects.exclude(
                id__in=ProgrammingGuide.objects.values_list("book_ptr_id", flat=True)
            )

        # Exclude Books that have the title "Don't index me!"
        indexed_objects = indexed_objects.exclude(title="Don't index me!")

        return indexed_objects

    def get_indexed_instance(self):
        # Check if this object is a Novel or ProgrammingGuide and return the specific object
        novel = Novel.objects.filter(book_ptr_id=self.id).first()
        programming_guide = ProgrammingGuide.objects.filter(book_ptr_id=self.id).first()

        # Return the novel/programming guide object if there is one, otherwise return self
        return novel or programming_guide or self

    def __str__(self):
        return self.title


class Character(models.Model):
    name = models.CharField(max_length=255)
    novel = models.ForeignKey(
        "Novel", related_name="characters", on_delete=models.CASCADE
    )

    def __str__(self):
        return self.name


class Novel(Book):
    setting = models.CharField(max_length=255)
    protagonist = models.OneToOneField(
        Character, related_name="+", null=True, on_delete=models.SET_NULL
    )

    search_fields = Book.search_fields + [
        index.SearchField("setting"),
        index.AutocompleteField("setting"),
        index.RelatedFields(
            "characters",
            [
                index.SearchField("name", boost=0.25),
            ],
        ),
        index.RelatedFields(
            "protagonist",
            [
                index.SearchField("name", boost=0.5),
                index.FilterField("novel"),
            ],
        ),
        index.FilterField("protagonist"),
    ]


class ProgrammingGuide(Book):
    programming_language = models.CharField(
        max_length=255,
        choices=[
            ("py", "Python"),
            ("js", "JavaScript"),
            ("rs", "Rust"),
        ],
    )

    search_fields = Book.search_fields + [
        index.SearchField("get_programming_language_display"),
        index.FilterField("programming_language"),
    ]


class UnindexedBook(index.Indexed, models.Model):
    title = models.CharField(max_length=255)
    publication_date = models.DateField()
    number_of_pages = models.IntegerField()
    tags = TaggableManager()

    search_fields = []



# Generated by Django 1.10.2 on 2016-10-07 15:33
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0001_initial"),
        ("wagtailcore", "0030_index_on_pagerevision_created_at"),
    ]

    operations = [
        migrations.AddField(
            model_name="pageviewrestriction",
            name="groups",
            field=models.ManyToManyField(blank=True, to="auth.Group"),
        ),
        migrations.AddField(
            model_name="pageviewrestriction",
            name="restriction_type",
            field=models.CharField(
                choices=[
                    ("none", "Public"),
                    ("login", "Private, accessible to logged-in users"),
                    ("password", "Private, accessible with the following password"),
                    ("groups", "Private, accessible to users in specific groups"),
                ],
                default="password",
                max_length=20,
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="pageviewrestriction",
            name="password",
            field=models.CharField(blank=True, max_length=255, verbose_name="password"),
        ),
    ]



from django.template import Context, Engine, TemplateSyntaxError, Variable
from django.test import TestCase

from wagtail.images.models import Image, Rendition
from wagtail.images.templatetags.wagtailimages_tags import ImageNode
from wagtail.images.tests.utils import (
    get_test_bad_image,
    get_test_image_file,
    get_test_image_file_svg,
    get_test_image_filename,
)

LIBRARIES = {
    "wagtailimages_tags": "wagtail.images.templatetags.wagtailimages_tags",
    "l10n": "django.templatetags.l10n",
}


class ImageNodeTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        # Create an image for running tests on
        cls.image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        cls.svg_image = Image.objects.create(
            title="Test SVG image",
            file=get_test_image_file_svg(),
        )

    def test_render_valid_image_to_string(self):
        """
        Tests that an ImageNode with a valid image renders an img tag
        """
        context = {"image": self.image}
        node = ImageNode(Variable("image"), ["original"])

        rendered = node.render(context)

        self.assertIn('<img alt="Test image"', rendered)

    def test_render_none_to_string(self):
        """
        Tests that an ImageNode without image renders an empty string
        """
        context = {"image": None}
        node = ImageNode(Variable("image"), ["original"])

        rendered = node.render(context)

        self.assertEqual(rendered, "")

    def test_render_valid_image_as_context_variable(self):
        """
        Tests that an ImageNode with a valid image and a context variable name
        renders an empty string and puts a rendition in the context variable
        """
        context = {"image": self.image, "image_node": "fake value"}
        node = ImageNode(Variable("image"), ["original"], "image_node")

        rendered = node.render(context)

        self.assertEqual(rendered, "")
        self.assertIsInstance(context["image_node"], Rendition)

    def test_render_none_as_context_variable(self):
        """
        Tests that an ImageNode without an image and a context variable name
        renders an empty string and puts None in the context variable
        """
        context = {"image": None, "image_node": "fake value"}
        node = ImageNode(Variable("image"), ["original"], "image_node")

        rendered = node.render(context)

        self.assertEqual(rendered, "")
        self.assertIsNone(context["image_node"])

    def test_filters_preserve_svg(self):
        """
        If the image is an SVG, and we set the preserve_svg parameter of ImageNode
        to True, we should only use filters that don't require rasterisation (at this
        time, resize and crop operations only).
        """
        params = [
            (self.svg_image, ["original"], "original"),
            (self.svg_image, ["fill-400x400", "bgcolor-000"], "fill-400x400"),
            (
                self.svg_image,
                ["fill-400x400", "format-webp", "webpquality-50"],
                "fill-400x400",
            ),
            (self.image, ["fill-400x400", "bgcolor-000"], "fill-400x400|bgcolor-000"),
            (self.image, ["fill-400x400", "format-webp"], "fill-400x400|format-webp"),
            (
                self.image,
                ["fill-400x400", "format-webp", "webpquality-50"],
                "fill-400x400|format-webp|webpquality-50",
            ),
            (self.svg_image, ["max-400x400"], "max-400x400"),
            (self.svg_image, ["min-400x400"], "min-400x400"),
            (self.svg_image, ["width-300"], "width-300"),
            (self.svg_image, ["height-300"], "height-300"),
            (self.svg_image, ["scale-50"], "scale-50"),
            (self.svg_image, ["fill-400x400"], "fill-400x400"),
            (self.svg_image, ["format-webp", "webpquality-50"], "original"),
        ]
        for image, filter_specs, expected in params:
            with self.subTest(img=image, filter_specs=filter_specs, expected=expected):
                context = {"image": image, "image_node": "fake_value"}
                node = ImageNode(
                    Variable("image"),
                    filter_specs + ["preserve-svg"],
                    output_var_name="rendition",
                )
                node.render(context)
                self.assertEqual(context["rendition"].filter_spec, expected)


class ImagesTestCase(TestCase):
    maxDiff = None

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.engine = Engine(
            app_dirs=True,
            libraries=LIBRARIES,
            builtins=[
                LIBRARIES["wagtailimages_tags"],
                LIBRARIES["l10n"],
            ],
        )

    @classmethod
    def setUpTestData(cls):
        # Create an image for running tests on
        cls.image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        cls.svg_image = Image.objects.create(
            title="Test SVG image",
            file=get_test_image_file_svg(),
        )
        cls.bad_image = get_test_bad_image()
        cls.bad_image.save()

    def render(self, string, context=None):
        if context is None:
            context = {}

        template = self.engine.from_string(string)
        return template.render(Context(context, autoescape=False))


class ImageTagTestCase(ImagesTestCase):
    def test_image(self):
        filename_200 = get_test_image_filename(self.image, "width-200")

        rendered = self.render("{% image myimage width-200 %}", {"myimage": self.image})
        self.assertHTMLEqual(
            rendered,
            f'<img alt="Test image" height="150" src="{filename_200}" width="200" />',
        )

    def test_none(self):
        rendered = self.render("{% image myimage width-2 %}", {"myimage": None})
        self.assertEqual(rendered, "")

    def test_missing_image(self):
        rendered = self.render(
            "{% image myimage width-200 %}", {"myimage": self.bad_image}
        )
        self.assertHTMLEqual(
            rendered,
            '<img alt="missing image" src="/media/not-found" width="0" height="0">',
        )

    def test_not_an_image(self):
        with self.assertRaisesMessage(
            ValueError, "Image template tags expect an Image object, got 'not a pipe'"
        ):
            self.render(
                "{% image myimage width-200 %}",
                {"myimage": "not a pipe"},
            )

    def test_invalid_character(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "filter specs in image tags may only"
        ):
            self.render(
                "{% image myimage fill-200×200 %}",
                {"myimage": self.image},
            )

    def test_multiple_as_variable(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "More than one variable name after 'as'"
        ):
            self.render(
                "{% image myimage width-200 as a b %}",
                {"myimage": self.image},
            )

    def test_missing_as_variable(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "Missing a variable name after 'as'"
        ):
            self.render(
                "{% image myimage width-200 as %}",
                {"myimage": self.image},
            )

    def test_mixing_as_variable_and_attrs(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "Do not use attributes with 'as' context assignments"
        ):
            self.render(
                "{% image myimage width-200 alt='Test' as test %}",
                {"myimage": self.image},
            )

    def test_missing_filter_spec(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "Image tags must be used with at least one filter spec"
        ):
            self.render(
                "{% image myimage %}",
                {"myimage": self.image},
            )


class SrcsetImageTagTestCase(ImagesTestCase):
    def test_srcset_image(self):
        filename_20 = get_test_image_filename(self.image, "width-20")
        filename_40 = get_test_image_filename(self.image, "width-40")

        rendered = self.render(
            "{% srcset_image myimage width-{20,40} sizes='100vw' %}",
            {"myimage": self.image},
        )
        expected = f"""
            <img
                sizes="100vw"
                src="{filename_20}"
                srcset="{filename_20} 20w, {filename_40} 40w"
                alt="Test image"
                width="20"
                height="15"
            >
        """
        self.assertHTMLEqual(rendered, expected)

    def test_srcset_output_single_image(self):
        self.assertHTMLEqual(
            self.render(
                "{% srcset_image myimage width-20 %}",
                {"myimage": self.image},
            ),
            self.render(
                "{% image myimage width-20 %}",
                {"myimage": self.image},
            ),
        )

    def test_none(self):
        rendered = self.render("{% srcset_image myimage width-2 %}", {"myimage": None})
        self.assertEqual(rendered, "")

    def test_invalid_character(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "filter specs in image tags may only contain"
        ):
            self.render(
                "{% srcset_image myimage fill-{200×200,400×400} sizes='100vw' %}",
                {"myimage": self.image},
            )

    def test_srcset_image_assignment(self):
        template = (
            "{% srcset_image myimage width-{30,60} as bg %}"
            "width: {{ bg.renditions.0.width|unlocalize }}, url: {{ bg.renditions.0.url }} "
            "width: {{ bg.renditions.1.width|unlocalize }}, url: {{ bg.renditions.1.url }} "
        )
        rendered = self.render(template, {"myimage": self.image})
        expected = f"""
            width: 30, url: {get_test_image_filename(self.image, "width-30")}
            width: 60, url: {get_test_image_filename(self.image, "width-60")}
        """
        self.assertHTMLEqual(rendered, expected)

    def test_srcset_image_assignment_render_as_is(self):
        filename_35 = get_test_image_filename(self.image, "width-35")
        filename_70 = get_test_image_filename(self.image, "width-70")

        rendered = self.render(
            "{% srcset_image myimage width-{35,70} as bg %}{{ bg }}",
            {"myimage": self.image},
        )
        expected = f"""
            <img
                src="{filename_35}"
                srcset="{filename_35} 35w, {filename_70} 70w"
                alt="Test image"
                width="35"
                height="26"
            >
        """
        self.assertHTMLEqual(rendered, expected)

    def test_missing_srcset_image(self):
        rendered = self.render(
            "{% srcset_image myimage width-{200,400} sizes='100vw' %}",
            {"myimage": self.bad_image},
        )
        expected = """
            <img
                sizes="100vw"
                src="/media/not-found"
                srcset="/media/not-found 0w, /media/not-found 0w"
                alt="missing image"
                width="0"
                height="0"
            >
        """
        self.assertHTMLEqual(rendered, expected)


class PictureTagTestCase(ImagesTestCase):
    def test_picture_formats_multi_sizes(self):
        filenames = [
            get_test_image_filename(self.image, "width-200.format-jpeg"),
            get_test_image_filename(self.image, "width-400.format-jpeg"),
            get_test_image_filename(self.image, "width-200.format-webp"),
            get_test_image_filename(self.image, "width-400.format-webp"),
            get_test_image_filename(self.image, "width-200.format-gif"),
            get_test_image_filename(self.image, "width-400.format-gif"),
        ]

        rendered = self.render(
            '{% picture myimage width-{200,400} format-{jpeg,webp,gif} sizes="100vw" %}',
            {"myimage": self.image},
        )
        expected = f"""
            <picture>
            <source srcset="{filenames[2]} 200w, {filenames[3]} 400w" sizes="100vw" type="image/webp">
            <source srcset="{filenames[0]} 200w, {filenames[1]} 400w" sizes="100vw" type="image/jpeg">
            <img
                sizes="100vw"
                src="{filenames[4]}"
                srcset="{filenames[4]} 200w, {filenames[5]} 400w"
                alt="Test image"
                width="200"
                height="150"
            >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)

    def test_picture_formats_only(self):
        filename_jpeg = get_test_image_filename(self.image, "format-jpeg")
        filename_webp = get_test_image_filename(self.image, "format-webp")

        rendered = self.render(
            "{% picture myimage format-{jpeg,webp} %}",
            {"myimage": self.image},
        )
        expected = f"""
            <picture>
            <source srcset="{filename_webp}" type="image/webp">
            <img
                src="{filename_jpeg}"
                alt="Test image"
                width="640"
                height="480"
            >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)

    def test_picture_sizes_only(self):
        rendered = self.render(
            '{% picture myimage width-{350,450} sizes="100vw" %}',
            {"myimage": self.image},
        )
        expected = self.render(
            '<picture>{% srcset_image myimage width-{350,450} sizes="100vw" %}</picture>',
            {"myimage": self.image},
        )
        self.assertHTMLEqual(rendered, expected)

    def test_picture_single_format(self):
        rendered = self.render(
            "{% picture myimage format-jpeg %}",
            {"myimage": self.image},
        )
        expected = self.render(
            "<picture>{% image myimage format-jpeg %}</picture>",
            {"myimage": self.image},
        )
        self.assertHTMLEqual(rendered, expected)

    def test_none(self):
        rendered = self.render("{% picture myimage width-2 %}", {"myimage": None})
        self.assertEqual(rendered, "")

    def test_picture_assignment(self):
        template = (
            "{% picture myimage width-{550,600} format-{jpeg,webp} as bg %}"
            "width: {{ bg.formats.jpeg.0.width|unlocalize }}, url: {{ bg.formats.jpeg.0.url }} "
            "width: {{ bg.formats.jpeg.1.width|unlocalize }}, url: {{ bg.formats.jpeg.1.url }} "
            "width: {{ bg.formats.webp.0.width|unlocalize }}, url: {{ bg.formats.webp.0.url }} "
            "width: {{ bg.formats.webp.1.width|unlocalize }}, url: {{ bg.formats.webp.1.url }} "
        )
        rendered = self.render(template, {"myimage": self.image})
        expected = f"""
            width: 550, url: {get_test_image_filename(self.image, "width-550.format-jpeg")}
            width: 600, url: {get_test_image_filename(self.image, "width-600.format-jpeg")}
            width: 550, url: {get_test_image_filename(self.image, "width-550.format-webp")}
            width: 600, url: {get_test_image_filename(self.image, "width-600.format-webp")}
        """
        self.assertHTMLEqual(rendered, expected)

    def test_picture_assignment_render_as_is(self):
        rendered = self.render(
            "{% picture myimage width-{2000,4000} format-{jpeg,webp} as bg %}{{ bg }}",
            {"myimage": self.image},
        )
        expected = self.render(
            "{% picture myimage width-{2000,4000} format-{jpeg,webp} %}",
            {"myimage": self.image},
        )
        self.assertHTMLEqual(rendered, expected)

    def test_missing_picture(self):
        rendered = self.render(
            "{% picture myimage width-{200,400} %}",
            {"myimage": self.bad_image},
        )
        expected = """
            <picture>
                <img
                    src="/media/not-found"
                    srcset="/media/not-found 0w, /media/not-found 0w"
                    alt="missing image"
                    width="0"
                    height="0"
                >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)

    def test_invalid_character(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "filter specs in image tags may only"
        ):
            self.render(
                '{% picture myimage fill-{20×20,40×40} sizes="100vw" %}',
                {"myimage": self.image},
            )

    def test_chaining_filterspecs(self):
        filename_jpeg = get_test_image_filename(
            self.image, "format-jpeg.jpegquality-40.webpquality-40"
        )
        filename_webp = get_test_image_filename(
            self.image, "format-webp.jpegquality-40.webpquality-40"
        )
        rendered = self.render(
            "{% picture myimage format-{jpeg,webp} jpegquality-40 webpquality-40 %}",
            {"myimage": self.image},
        )
        expected = f"""
            <picture>
            <source srcset="{filename_webp}" type="image/webp">
            <img
                src="{filename_jpeg}"
                alt="Test image"
                width="640"
                height="480"
            >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)

    def test_preserve_svg_with_raster_image(self):
        filenames = [
            get_test_image_filename(self.image, "width-180.format-avif"),
            get_test_image_filename(self.image, "width-360.format-avif"),
            get_test_image_filename(self.image, "width-180.format-jpeg"),
            get_test_image_filename(self.image, "width-360.format-jpeg"),
        ]

        rendered = self.render(
            '{% picture myimage width-{180,360} format-{avif,jpeg} preserve-svg sizes="100vw" %}',
            {"myimage": self.image},
        )
        expected = f"""
            <picture>
            <source srcset="{filenames[0]} 180w, {filenames[1]} 360w" sizes="100vw" type="image/avif">
            <img
                sizes="100vw"
                src="{filenames[2]}"
                srcset="{filenames[2]} 180w, {filenames[3]} 360w"
                alt="Test image"
                width="180"
                height="135"
            >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)

    def test_preserve_svg_with_svg_image(self):
        filenames = [
            get_test_image_filename(self.svg_image, "width-40"),
            get_test_image_filename(self.svg_image, "width-80"),
        ]

        rendered = self.render(
            '{% picture myimage width-{40,80} format-{avif,jpeg} preserve-svg sizes="100vw" %}',
            {"myimage": self.svg_image},
        )
        expected = f"""
            <picture>
            <img
                sizes="100vw"
                src="{filenames[0]}"
                srcset="{filenames[0]} 40.0w, {filenames[1]} 80.0w"
                alt="Test SVG image"
                width="40.0"
                height="40.0"
            >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)



from django.apps import apps
from django.contrib.admin.utils import unquote
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.http import Http404, JsonResponse
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.utils import timezone
from django.views.decorators.http import require_POST

from wagtail.admin.models import EditingSession
from wagtail.admin.ui.editing_sessions import EditingSessionsList
from wagtail.admin.utils import get_user_display_name
from wagtail.models import Page, Revision, RevisionMixin, WorkflowMixin


@require_POST
def ping(request, app_label, model_name, object_id, session_id):
    try:
        model = apps.get_model(app_label, model_name)
    except LookupError:
        raise Http404

    unquoted_object_id = unquote(object_id)

    content_type = ContentType.objects.get_for_model(model)

    obj = get_object_or_404(model, pk=unquoted_object_id)
    if isinstance(obj, Page):
        can_edit = obj.permissions_for_user(request.user).can_edit()
    else:
        try:
            permission_policy = model.snippet_viewset.permission_policy
        except AttributeError:
            # model is neither a Page nor a snippet
            raise Http404

        can_edit = permission_policy.user_has_permission_for_instance(
            request.user, "change", obj
        )
        if not can_edit and isinstance(obj, WorkflowMixin):
            workflow = obj.get_workflow()
            if workflow is not None:
                current_workflow_task = obj.current_workflow_task
                can_edit = (
                    current_workflow_task
                    and current_workflow_task.user_can_access_editor(obj, request.user)
                )

    if not can_edit:
        raise Http404

    try:
        session = EditingSession.objects.get(
            id=session_id,
            user=request.user,
            content_type=content_type,
            object_id=unquoted_object_id,
        )
    except EditingSession.DoesNotExist:
        session = EditingSession(
            content_type=content_type,
            object_id=unquoted_object_id,
            user=request.user,
        )

    session.last_seen_at = timezone.now()
    session.is_editing = request.POST.get("is_editing", False)
    try:
        session.full_clean()
    except ValidationError:
        return JsonResponse({"error": "Invalid data"}, status=400)
    else:
        session.save()

    other_sessions = (
        EditingSession.objects.filter(
            content_type=content_type,
            object_id=unquoted_object_id,
            last_seen_at__gte=timezone.now() - timezone.timedelta(minutes=1),
        )
        .exclude(id=session.id)
        .select_related("user", "user__wagtail_userprofile")
        .order_by("-last_seen_at")
    )

    # create a lookup of sessions indexed by user ID. Multiple sessions from the same user
    # are merged, such that the most recently seen one is reported, but is_editing is true
    # if any session has the editing flag set (not just the latest one).
    other_sessions_lookup = {}
    for other_session in other_sessions:
        try:
            other_session_info = other_sessions_lookup[other_session.user.pk]
        except KeyError:
            other_sessions_lookup[other_session.user.pk] = {
                "session_id": other_session.id,
                "user": other_session.user,
                "last_seen_at": other_session.last_seen_at,
                "is_editing": other_session.is_editing,
                "revision_id": None,
            }
        else:
            if other_session.is_editing:
                other_session_info["is_editing"] = True

    revision_id = request.POST.get("revision_id", None)
    if revision_id is not None and issubclass(model, RevisionMixin):
        all_revisions = obj.revisions.defer("content")
        try:
            original_revision = all_revisions.get(id=revision_id)
        except Revision.DoesNotExist:
            raise Http404

        newest_revision = (
            all_revisions.filter(created_at__gt=original_revision.created_at)
            .order_by("-created_at", "-pk")
            .select_related("user")
            .first()
        )

        if newest_revision:
            try:
                session_info = other_sessions_lookup[newest_revision.user_id]
            except KeyError:
                other_sessions_lookup[newest_revision.user_id] = {
                    "session_id": None,
                    "user": newest_revision.user,
                    "last_seen_at": newest_revision.created_at,
                    "is_editing": False,
                    "revision_id": newest_revision.id,
                }
            else:
                session_info["revision_id"] = newest_revision.id
                if newest_revision.created_at > session_info["last_seen_at"]:
                    session_info["last_seen_at"] = newest_revision.created_at

    try:
        users_other_session = other_sessions_lookup[request.user.pk]
    except KeyError:
        pass
    else:
        # If the user has a different session that is not editing and hasn't
        # created the latest revision, hide it as it's not relevant.
        if (
            not users_other_session["is_editing"]
            and not users_other_session["revision_id"]
        ):
            other_sessions_lookup.pop(request.user.pk)

    # Sort the other sessions so that they are presented in the following order:
    # 1. Prioritise any session with the latest revision. Then,
    # 2. Prioritise any session that is currently editing. Then,
    # 3. Prioritise any session with the smallest id, so that new sessions are
    #    appended to the end of the list (they're shown last). We are not sorting
    #    by last_seen_at to avoid shifting the order of the sessions as they
    #    ping the server.
    other_sessions = sorted(
        other_sessions_lookup.values(),
        key=lambda other_session: (
            # We want to sort revision_id and is_editing in descending order,
            # but we want to sort session_id in ascending order. To achieve this
            # in a single pass, we negate the values of revision_id and is_editing.
            # We can negate revision_id because there can only be one (at most)
            # session with revision_id, so we only care about the presence and
            # not the ID itself, thus we can treat it as a boolean flag.
            not other_session["revision_id"],
            not other_session["is_editing"],
            other_session["session_id"],
        ),
    )

    return JsonResponse(
        {
            "session_id": session.id,
            "ping_url": reverse(
                "wagtailadmin_editing_sessions:ping",
                args=(app_label, model_name, object_id, session.id),
            ),
            "release_url": reverse(
                "wagtailadmin_editing_sessions:release", args=(session.id,)
            ),
            "other_sessions": [
                {
                    "session_id": other_session["session_id"],
                    "user": get_user_display_name(other_session["user"]),
                    "last_seen_at": other_session["last_seen_at"].isoformat(),
                    "is_editing": other_session["is_editing"],
                    "revision_id": other_session["revision_id"],
                }
                for other_session in other_sessions
            ],
            "html": EditingSessionsList(session, other_sessions).render_html(),
        }
    )


@require_POST
def release(request, session_id):
    EditingSession.objects.filter(id=session_id, user=request.user).delete()
    return JsonResponse({})



import json
import re

from django import template
from django.core.cache import cache
from django.core.cache.utils import make_template_fragment_key
from django.http import HttpRequest
from django.template import TemplateSyntaxError, VariableDoesNotExist
from django.test import TestCase
from django.test.utils import override_settings
from django.urls.exceptions import NoReverseMatch
from django.utils.safestring import SafeString
from django.utils.translation import gettext_lazy

from wagtail.coreutils import (
    get_dummy_request,
    get_js_regex,
    make_wagtail_template_fragment_key,
    resolve_model_string,
)
from wagtail.models import Locale, Page, Site, SiteRootPath
from wagtail.models.sites import (
    SITE_ROOT_PATHS_CACHE_KEY,
    SITE_ROOT_PATHS_CACHE_VERSION,
)
from wagtail.templatetags.wagtail_cache import WagtailPageCacheNode
from wagtail.templatetags.wagtailcore_tags import richtext, slugurl
from wagtail.test.testapp.models import SimplePage


class TestPageUrlTags(TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        super().setUp()

        # Clear caches
        cache.clear()

    def test_pageurl_tag(self):
        response = self.client.get("/events/")
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<a href="/events/christmas/">Christmas</a>')

    def test_pageurl_with_named_url_fallback(self):
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% pageurl page fallback='fallback' %}">Fallback</a>"""
        )
        with self.assertNumQueries(0):
            result = tpl.render(template.Context({"page": None}))
        self.assertIn('<a href="/fallback/">Fallback</a>', result)

    def test_pageurl_with_get_absolute_url_object_fallback(self):
        class ObjectWithURLMethod:
            def get_absolute_url(self):
                return "/object-specific-url/"

        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% pageurl page fallback=object_with_url_method %}">Fallback</a>"""
        )
        result = tpl.render(
            template.Context(
                {"page": None, "object_with_url_method": ObjectWithURLMethod()}
            )
        )
        self.assertIn('<a href="/object-specific-url/">Fallback</a>', result)

    def test_pageurl_with_valid_url_string_fallback(self):
        """
        `django.shortcuts.resolve_url` accepts strings containing '.' or '/' as they are.
        """
        tpl = template.Template(
            """
            {% load wagtailcore_tags %}
            <a href="{% pageurl page fallback='.' %}">Same page fallback</a>
            <a href="{% pageurl page fallback='/' %}">Homepage fallback</a>
            <a href="{% pageurl page fallback='../' %}">Up one step fallback</a>
            """
        )
        result = tpl.render(template.Context({"page": None}))
        self.assertIn('<a href=".">Same page fallback</a>', result)
        self.assertIn('<a href="/">Homepage fallback</a>', result)
        self.assertIn('<a href="../">Up one step fallback</a>', result)

    def test_pageurl_with_invalid_url_string_fallback(self):
        """
        Strings not containing '.' or '/', and not matching a named URL will error.
        """
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% pageurl page fallback='not-existing-endpoint' %}">Fallback</a>"""
        )
        with self.assertRaises(NoReverseMatch):
            tpl.render(template.Context({"page": None}))

    def test_slugurl_tag(self):
        response = self.client.get("/events/christmas/")
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<a href="/events/">Back to events index</a>')

    def test_pageurl_without_request_in_context(self):
        page = Page.objects.get(url_path="/home/events/")
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% pageurl page %}">{{ page.title }}</a>"""
        )

        # no 'request' object in context
        with self.assertNumQueries(7):
            result = tpl.render(template.Context({"page": page}))
        self.assertIn('<a href="/events/">Events</a>', result)

        # 'request' object in context, but no 'site' attribute
        result = tpl.render(
            template.Context({"page": page, "request": get_dummy_request()})
        )
        self.assertIn('<a href="/events/">Events</a>', result)

    def test_pageurl_caches(self):
        page = Page.objects.get(url_path="/home/events/")
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% pageurl page %}">{{ page.title }}</a>"""
        )

        request = get_dummy_request()

        with self.assertNumQueries(8):
            result = tpl.render(template.Context({"page": page, "request": request}))
        self.assertIn('<a href="/events/">Events</a>', result)

        with self.assertNumQueries(0):
            result = tpl.render(template.Context({"page": page, "request": request}))
        self.assertIn('<a href="/events/">Events</a>', result)

    @override_settings(ALLOWED_HOSTS=["testserver", "localhost", "unknown.example.com"])
    def test_pageurl_with_unknown_site(self):
        page = Page.objects.get(url_path="/home/events/")
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% pageurl page %}">{{ page.title }}</a>"""
        )

        # 'request' object in context, but site is None
        request = get_dummy_request()
        request.META["HTTP_HOST"] = "unknown.example.com"
        with self.assertNumQueries(8):
            result = tpl.render(template.Context({"page": page, "request": request}))
        self.assertIn('<a href="/events/">Events</a>', result)

    def test_bad_pageurl(self):
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% pageurl page %}">{{ page.title }}</a>"""
        )

        with self.assertRaisesRegex(
            ValueError, "pageurl tag expected a Page object, got None"
        ):
            tpl.render(template.Context({"page": None}))

    def test_bad_slugurl(self):
        # no 'request' object in context
        result = slugurl(template.Context({}), "bad-slug-doesnt-exist")
        self.assertIsNone(result)

        # 'request' object in context, but no 'site' attribute
        result = slugurl(
            context=template.Context({"request": HttpRequest()}),
            slug="bad-slug-doesnt-exist",
        )
        self.assertIsNone(result)

    @override_settings(ALLOWED_HOSTS=["testserver", "localhost", "site2.example.com"])
    def test_slugurl_tag_returns_url_for_current_site(self):
        home_page = Page.objects.get(url_path="/home/")
        new_home_page = home_page.copy(
            update_attrs={"title": "New home page", "slug": "new-home"}
        )
        second_site = Site.objects.create(
            hostname="site2.example.com", root_page=new_home_page
        )
        # Add a page to the new site that has a slug that is the same as one on
        # the first site, but is in a different position in the treeself.
        new_christmas_page = Page(title="Christmas", slug="christmas")
        new_home_page.add_child(instance=new_christmas_page)
        request = get_dummy_request(site=second_site)
        url = slugurl(context=template.Context({"request": request}), slug="christmas")
        self.assertEqual(url, "/christmas/")

    @override_settings(ALLOWED_HOSTS=["testserver", "localhost", "site2.example.com"])
    def test_slugurl_tag_returns_url_for_other_site(self):
        home_page = Page.objects.get(url_path="/home/")
        new_home_page = home_page.copy(
            update_attrs={"title": "New home page", "slug": "new-home"}
        )
        second_site = Site.objects.create(
            hostname="site2.example.com", root_page=new_home_page
        )
        request = get_dummy_request(site=second_site)
        # There is no page with this slug on the current site, so this
        # should return an absolute URL for the page on the first site.
        url = slugurl(slug="christmas", context=template.Context({"request": request}))
        self.assertEqual(url, "http://localhost/events/christmas/")

    def test_slugurl_without_request_in_context(self):
        # no 'request' object in context
        result = slugurl(template.Context({}), "events")
        self.assertEqual(result, "/events/")

        # 'request' object in context, but no 'site' attribute
        with self.assertNumQueries(3):
            result = slugurl(
                template.Context({"request": get_dummy_request()}), "events"
            )
        self.assertEqual(result, "/events/")

    @override_settings(ALLOWED_HOSTS=["testserver", "localhost", "unknown.example.com"])
    def test_slugurl_with_null_site_in_request(self):
        # 'request' object in context, but site is None
        request = get_dummy_request()
        request.META["HTTP_HOST"] = "unknown.example.com"
        result = slugurl(template.Context({"request": request}), "events")
        self.assertEqual(result, "/events/")

    def test_fullpageurl(self):
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% fullpageurl page %}">Events</a>"""
        )
        page = Page.objects.get(url_path="/home/events/")
        with self.assertNumQueries(7):
            result = tpl.render(template.Context({"page": page}))
        self.assertIn('<a href="http://localhost/events/">Events</a>', result)

    def test_fullpageurl_with_named_url_fallback(self):
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% fullpageurl page fallback='fallback' %}">Fallback</a>"""
        )
        with self.assertNumQueries(0):
            result = tpl.render(template.Context({"page": None}))
        self.assertIn('<a href="/fallback/">Fallback</a>', result)

    def test_fullpageurl_with_absolute_fallback(self):
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% fullpageurl page fallback='fallback' %}">Fallback</a>"""
        )
        with self.assertNumQueries(0):
            result = tpl.render(
                template.Context({"page": None, "request": get_dummy_request()})
            )
        self.assertIn('<a href="http://localhost/fallback/">Fallback</a>', result)

    def test_fullpageurl_with_invalid_page(self):
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% fullpageurl page %}">Events</a>"""
        )
        with self.assertRaises(ValueError):
            tpl.render(template.Context({"page": 123}))

    def test_pageurl_with_invalid_page(self):
        tpl = template.Template(
            """{% load wagtailcore_tags %}<a href="{% pageurl page %}">Events</a>"""
        )
        with self.assertRaises(ValueError):
            tpl.render(template.Context({"page": 123}))


class TestWagtailSiteTag(TestCase):
    fixtures = ["test.json"]

    def test_wagtail_site_tag(self):
        request = get_dummy_request(site=Site.objects.first())

        tpl = template.Template(
            """{% load wagtailcore_tags %}{% wagtail_site as current_site %}{{ current_site.hostname }}"""
        )
        result = tpl.render(template.Context({"request": request}))
        self.assertEqual("localhost", result)

    def test_wagtail_site_tag_with_missing_request_context(self):
        tpl = template.Template(
            """{% load wagtailcore_tags %}{% wagtail_site as current_site %}{{ current_site.hostname }}"""
        )
        result = tpl.render(template.Context({}))
        # should fail silently
        self.assertEqual("", result)


class TestSiteRootPathsCache(TestCase):
    fixtures = ["test.json"]

    def get_cached_site_root_paths(self):
        return cache.get(
            SITE_ROOT_PATHS_CACHE_KEY, version=SITE_ROOT_PATHS_CACHE_VERSION
        )

    def test_cache(self):
        """
        This tests that the cache is populated when building URLs
        """
        # Get homepage
        homepage = Page.objects.get(url_path="/home/")

        # Warm up the cache by getting the url
        _ = homepage.url

        # Check that the cache has been set correctly
        self.assertEqual(
            self.get_cached_site_root_paths(),
            [
                SiteRootPath(
                    site_id=1,
                    root_path="/home/",
                    root_url="http://localhost",
                    language_code="en",
                )
            ],
        )

    def test_cache_backend_uses_json_serialization(self):
        """
        This tests that, even if the cache backend uses JSON serialization,
        get_site_root_paths() returns a list of SiteRootPath objects.
        """
        result = Site.get_site_root_paths()

        self.assertEqual(
            result,
            [
                SiteRootPath(
                    site_id=1,
                    root_path="/home/",
                    root_url="http://localhost",
                    language_code="en",
                )
            ],
        )

        # Go through JSON (de)serialisation to check that the result is
        # still a list of named tuples.
        cache.set(
            SITE_ROOT_PATHS_CACHE_KEY,
            json.loads(json.dumps(result)),
            version=SITE_ROOT_PATHS_CACHE_VERSION,
        )

        result = Site.get_site_root_paths()
        self.assertIsInstance(result[0], SiteRootPath)

    def test_cache_clears_when_site_saved(self):
        """
        This tests that the cache is cleared whenever a site is saved
        """
        # Get homepage
        homepage = Page.objects.get(url_path="/home/")

        # Warm up the cache by getting the url
        _ = homepage.url

        # Check that the cache has been set
        self.assertEqual(
            self.get_cached_site_root_paths(),
            [
                SiteRootPath(
                    site_id=1,
                    root_path="/home/",
                    root_url="http://localhost",
                    language_code="en",
                )
            ],
        )

        # Save the site
        Site.objects.get(is_default_site=True).save()

        # Check that the cache has been cleared
        self.assertIsNone(self.get_cached_site_root_paths())

    def test_cache_clears_when_site_deleted(self):
        """
        This tests that the cache is cleared whenever a site is deleted
        """
        # Get homepage
        homepage = Page.objects.get(url_path="/home/")

        # Warm up the cache by getting the url
        _ = homepage.url

        # Check that the cache has been set
        self.assertEqual(
            self.get_cached_site_root_paths(),
            [
                SiteRootPath(
                    site_id=1,
                    root_path="/home/",
                    root_url="http://localhost",
                    language_code="en",
                )
            ],
        )

        # Delete the site
        Site.objects.get(is_default_site=True).delete()

        # Check that the cache has been cleared
        self.assertIsNone(self.get_cached_site_root_paths())

    def test_cache_clears_when_site_root_moves(self):
        """
        This tests for an issue where if a site root page was moved, all
        the page urls in that site would change to None.

        The issue was caused by the 'wagtail_site_root_paths' cache
        variable not being cleared when a site root page was moved. Which
        left all the child pages thinking that they are no longer in the
        site and return None as their url.

        Fix: d6cce69a397d08d5ee81a8cbc1977ab2c9db2682
        Discussion: https://github.com/wagtail/wagtail/issues/7
        """
        # Get homepage, root page and site
        root_page = Page.objects.get(id=1)
        homepage = Page.objects.get(url_path="/home/")
        default_site = Site.objects.get(is_default_site=True)

        # Create a new homepage under current homepage
        new_homepage = SimplePage(
            title="New Homepage", slug="new-homepage", content="hello"
        )
        homepage.add_child(instance=new_homepage)

        # Set new homepage as the site root page
        default_site.root_page = new_homepage
        default_site.save()

        # Warm up the cache by getting the url
        _ = homepage.url

        # Move new homepage to root
        new_homepage.move(root_page, pos="last-child")

        # Get fresh instance of new_homepage
        new_homepage = Page.objects.get(id=new_homepage.id)

        # Check url
        self.assertEqual(new_homepage.url, "/")

    def test_cache_clears_when_site_root_slug_changes(self):
        """
        This tests for an issue where if a site root pages slug was
        changed, all the page urls in that site would change to None.

        The issue was caused by the 'wagtail_site_root_paths' cache
        variable not being cleared when a site root page was changed.
        Which left all the child pages thinking that they are no longer in
        the site and return None as their url.

        Fix: d6cce69a397d08d5ee81a8cbc1977ab2c9db2682
        Discussion: https://github.com/wagtail/wagtail/issues/157
        """
        # Get homepage
        homepage = Page.objects.get(url_path="/home/")

        # Warm up the cache by getting the url
        _ = homepage.url

        # Change homepage title and slug
        homepage.title = "New home"
        homepage.slug = "new-home"
        homepage.save()

        # Get fresh instance of homepage
        homepage = Page.objects.get(id=homepage.id)

        # Check url
        self.assertEqual(homepage.url, "/")

    @override_settings(WAGTAIL_I18N_ENABLED=True)
    def test_cache_clears_when_site_root_is_translated_as_alias(self):
        # Get homepage
        homepage = Page.objects.get(url_path="/home/")

        # Warm up the cache by getting the url
        _ = homepage.url

        # Translate the homepage
        translated_homepage = homepage.copy_for_translation(
            Locale.objects.create(language_code="fr"), alias=True
        )

        # Check url
        self.assertEqual(translated_homepage.url, "/")


class TestResolveModelString(TestCase):
    def test_resolve_from_string(self):
        model = resolve_model_string("wagtailcore.Page")

        self.assertEqual(model, Page)

    def test_resolve_from_string_with_default_app(self):
        model = resolve_model_string("Page", default_app="wagtailcore")

        self.assertEqual(model, Page)

    def test_resolve_from_string_with_different_default_app(self):
        model = resolve_model_string("wagtailcore.Page", default_app="wagtailadmin")

        self.assertEqual(model, Page)

    def test_resolve_from_class(self):
        model = resolve_model_string(Page)

        self.assertEqual(model, Page)

    def test_resolve_from_string_invalid(self):
        self.assertRaises(ValueError, resolve_model_string, "wagtail.core.Page")

    def test_resolve_from_string_with_incorrect_default_app(self):
        self.assertRaises(
            LookupError, resolve_model_string, "Page", default_app="wagtailadmin"
        )

    def test_resolve_from_string_with_unknown_model_string(self):
        self.assertRaises(LookupError, resolve_model_string, "wagtailadmin.Page")

    def test_resolve_from_string_with_no_default_app(self):
        self.assertRaises(ValueError, resolve_model_string, "Page")

    def test_resolve_from_class_that_isnt_a_model(self):
        model = resolve_model_string(object)
        self.assertEqual(model, object)

    def test_resolve_from_bad_type(self):
        self.assertRaises(ValueError, resolve_model_string, resolve_model_string)

    def test_resolve_from_none(self):
        self.assertRaises(ValueError, resolve_model_string, None)


class TestRichtextTag(TestCase):
    def test_call_with_text(self):
        result = richtext("Hello world!")
        self.assertEqual(result, "Hello world!")
        self.assertIsInstance(result, SafeString)

    def test_call_with_lazy(self):
        result = richtext(gettext_lazy("test"))
        self.assertEqual(result, "test")

    def test_call_with_none(self):
        result = richtext(None)
        self.assertEqual(result, "")

    def test_call_with_invalid_value(self):
        with self.assertRaisesRegex(
            TypeError, "'richtext' template filter received an invalid value"
        ):
            richtext(42)

    def test_call_with_bytes(self):
        with self.assertRaisesRegex(
            TypeError, "'richtext' template filter received an invalid value"
        ):
            richtext(b"Hello world!")


class TestWagtailCacheTag(TestCase):
    def setUp(self):
        cache.clear()

    def test_caches(self):
        request = get_dummy_request()
        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailcache 100 test %}{{ foo.bar }}{% endwagtailcache %}"""
        )

        result = tpl.render(
            template.Context({"request": request, "foo": {"bar": "foobar"}})
        )
        self.assertEqual(result, "foobar")

        result2 = tpl.render(
            template.Context({"request": request, "foo": {"bar": "baz"}})
        )
        self.assertEqual(result2, "foobar")

        self.assertEqual(cache.get(make_template_fragment_key("test")), "foobar")

    def test_caches_on_additional_parameters(self):
        request = get_dummy_request()
        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailcache 100 test foo %}{{ foo.bar }}{% endwagtailcache %}"""
        )

        result = tpl.render(
            template.Context({"request": request, "foo": {"bar": "foobar"}})
        )
        self.assertEqual(result, "foobar")

        result2 = tpl.render(
            template.Context({"request": request, "foo": {"bar": "baz"}})
        )
        self.assertEqual(result2, "baz")

        self.assertEqual(
            cache.get(make_template_fragment_key("test", [{"bar": "foobar"}])), "foobar"
        )
        self.assertEqual(
            cache.get(make_template_fragment_key("test", [{"bar": "baz"}])), "baz"
        )

    def test_skips_cache_in_preview(self):
        request = get_dummy_request()
        request.is_preview = True

        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailcache 100 test %}{{ foo.bar }}{% endwagtailcache %}"""
        )

        result = tpl.render(
            template.Context({"request": request, "foo": {"bar": "foobar"}})
        )
        self.assertEqual(result, "foobar")

        result2 = tpl.render(
            template.Context({"request": request, "foo": {"bar": "baz"}})
        )
        self.assertEqual(result2, "baz")

        self.assertIsNone(cache.get(make_template_fragment_key("test")))

    def test_no_request(self):
        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailcache 100 test %}{{ foo.bar }}{% endwagtailcache %}"""
        )

        result = tpl.render(template.Context({"foo": {"bar": "foobar"}}))
        self.assertEqual(result, "foobar")

        result2 = tpl.render(template.Context({"foo": {"bar": "baz"}}))
        self.assertEqual(result2, "baz")

        self.assertIsNone(cache.get(make_template_fragment_key("test")))  #

    def test_invalid_usage(self):
        with self.assertRaises(TemplateSyntaxError) as e:
            template.Template(
                """{% load wagtail_cache %}{% wagtailcache 100 %}{{ foo.bar }}{% endwagtailcache %}"""
            )
        self.assertEqual(
            e.exception.args[0], "'wagtailcache' tag requires at least 2 arguments."
        )


class TestWagtailPageCacheTag(TestCase):
    fixtures = ["test.json"]

    @classmethod
    def setUpTestData(cls):
        cls.page_1 = Page.objects.first()
        cls.page_2 = Page.objects.all()[2]
        cls.site = Site.objects.get(hostname="localhost", port=80)

    def test_caches(self):
        request = get_dummy_request(site=self.site)
        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailpagecache 100 test %}{{ foo.bar }}{% endwagtailpagecache %}"""
        )

        result = tpl.render(
            template.Context(
                {"request": request, "foo": {"bar": "foobar"}, "page": self.page_1}
            )
        )
        self.assertEqual(result, "foobar")

        result2 = tpl.render(
            template.Context(
                {"request": request, "foo": {"bar": "baz"}, "page": self.page_1}
            )
        )
        self.assertEqual(result2, "foobar")

        self.assertEqual(
            cache.get(
                make_wagtail_template_fragment_key("test", self.page_1, self.site)
            ),
            "foobar",
        )

    def test_caches_additional_parameters(self):
        request = get_dummy_request(site=self.site)
        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailpagecache 100 test foo %}{{ foo.bar }}{% endwagtailpagecache %}"""
        )

        result = tpl.render(
            template.Context(
                {"request": request, "foo": {"bar": "foobar"}, "page": self.page_1}
            )
        )
        self.assertEqual(result, "foobar")

        result2 = tpl.render(
            template.Context(
                {"request": request, "foo": {"bar": "baz"}, "page": self.page_1}
            )
        )
        self.assertEqual(result2, "baz")

        self.assertEqual(
            cache.get(
                make_wagtail_template_fragment_key(
                    "test", self.page_1, self.site, [{"bar": "foobar"}]
                )
            ),
            "foobar",
        )
        self.assertEqual(
            cache.get(
                make_wagtail_template_fragment_key(
                    "test", self.page_1, self.site, [{"bar": "baz"}]
                )
            ),
            "baz",
        )

    def test_doesnt_pollute_cache(self):
        request = get_dummy_request(site=self.site)
        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailpagecache 100 test %}{{ foo.bar }}{% endwagtailpagecache %}"""
        )

        context = template.Context(
            {"request": request, "foo": {"bar": "foobar"}, "page": self.page_1}
        )
        result = tpl.render(context)
        self.assertEqual(result, "foobar")

        self.assertNotIn(WagtailPageCacheNode.CACHE_SITE_TEMPLATE_VAR, context)

    def test_skips_cache_in_preview(self):
        request = get_dummy_request(site=self.site)
        request.is_preview = True

        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailpagecache 100 test %}{{ foo.bar }}{% endwagtailpagecache %}"""
        )

        result = tpl.render(
            template.Context(
                {"request": request, "foo": {"bar": "foobar"}, "page": self.page_1}
            )
        )
        self.assertEqual(result, "foobar")

        result2 = tpl.render(
            template.Context(
                {"request": request, "foo": {"bar": "baz"}, "page": self.page_1}
            )
        )
        self.assertEqual(result2, "baz")

        self.assertIsNone(
            cache.get(
                make_wagtail_template_fragment_key("test", self.page_1, self.site)
            )
        )

    def test_no_request(self):
        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailpagecache 100 test %}{{ foo.bar }}{% endwagtailpagecache %}"""
        )

        result = tpl.render(
            template.Context({"foo": {"bar": "foobar"}, "page": self.page_1})
        )
        self.assertEqual(result, "foobar")

        result2 = tpl.render(
            template.Context({"foo": {"bar": "baz"}, "page": self.page_1})
        )
        self.assertEqual(result2, "baz")

        self.assertIsNone(
            cache.get(
                make_wagtail_template_fragment_key("test", self.page_1, self.site)
            )
        )

    def test_no_page(self):
        request = get_dummy_request()

        tpl = template.Template(
            """{% load wagtail_cache %}{% wagtailpagecache 100 test %}{{ foo.bar }}{% endwagtailpagecache %}"""
        )

        with self.assertRaises(VariableDoesNotExist) as e:
            tpl.render(template.Context({"request": request, "foo": {"bar": "foobar"}}))

        self.assertEqual(e.exception.params[0], "page")

    def test_cache_key(self):
        self.assertEqual(
            make_wagtail_template_fragment_key("test", self.page_1, self.site),
            make_template_fragment_key(
                "test", vary_on=[self.page_1.cache_key, self.site.id]
            ),
        )

    def test_invalid_usage(self):
        with self.assertRaises(TemplateSyntaxError) as e:
            template.Template(
                """{% load wagtail_cache %}{% wagtailpagecache 100 %}{{ foo.bar }}{% endwagtailpagecache %}"""
            )
        self.assertEqual(
            e.exception.args[0], "'wagtailpagecache' tag requires at least 2 arguments."
        )


class TestRegexJavaScriptConversion(TestCase):
    """
    Tests for the get_js_regex function that converts Python regex to JavaScript regex.
    """

    def test_empty_value(self):
        self.assertEqual(get_js_regex(), [])
        self.assertEqual(get_js_regex(None), [])
        self.assertEqual(get_js_regex(""), [])

    def test_compiled_regex(self):
        self.assertEqual(get_js_regex(re.compile(r"\D")), ["\\D", "gu"])

    def test_string(self):
        self.assertEqual(get_js_regex(r"\D"), ["\\D", "gu"])

    def test_regex_with_flags(self):
        self.assertEqual(
            get_js_regex(re.compile(r"w{1,3}", re.IGNORECASE)), ["w{1,3}", "giu"]
        )
        self.assertEqual(
            get_js_regex(re.compile(r".*", re.MULTILINE | re.S)), [".*", "gmsu"]
        )

    def test_custom_base_js_flags(self):
        self.assertEqual(get_js_regex(r"\D", base_js_flags="g"), ["\\D", "g"])
        self.assertEqual(
            get_js_regex(re.compile(r"\D", re.MULTILINE), base_js_flags="g"),
            ["\\D", "gm"],
        )
        self.assertEqual(get_js_regex(r"\D", base_js_flags=""), ["\\D", ""])

    def test_regex_with_inline_flags(self):
        self.assertEqual(get_js_regex(r"(?i)\D"), ["\\D", "giu"])
        self.assertEqual(
            get_js_regex(re.compile(r"(?i)\D", re.MULTILINE)), ["\\D", "gimu"]
        )

    def test_regex_with_invalid_flag_locale(self):
        """
        Using the re.LOCALE flag is discouraged in Python docs & not supported in JavaScript
        """

        with self.assertRaises(ValueError) as error:
            get_js_regex(re.compile(rb"\w", re.LOCALE))

        self.assertEqual(
            str(error.exception),
            "Python re.LOCALE flag is not supported in JavaScript.",
        )

        with self.assertRaises(ValueError) as error:
            get_js_regex(rb"(?L)\w")

        self.assertEqual(
            str(error.exception),
            "Python re.LOCALE flag is not supported in JavaScript.",
        )

    def test_regex_with_invalid_flag_verbose(self):
        """
        Until we do advanced cleaning on the Verbose style of regex
        we will not support it in JavaScript.
        """

        with self.assertRaises(ValueError) as error:
            get_js_regex(r"(?x)\d{ 3 }")

        self.assertEqual(
            str(error.exception),
            "Python re.VERBOSE flag is not supported in JavaScript.",
        )

        with self.assertRaises(ValueError) as error:
            verbose_regex = re.compile(
                r"""
                \d+  # match one or more digits
                \s*  # match zero or more whitespace characters
                \w+  # match one or more word characters
            """,
                re.X,
            )
            get_js_regex(verbose_regex)

        self.assertEqual(
            str(error.exception),
            "Python re.VERBOSE flag is not supported in JavaScript.",
        )



from django.test import TestCase
from django.urls import reverse
from django.utils import timezone

from wagtail.test.utils.wagtail_tests import WagtailTestUtils


class TestBaseViewSet(WagtailTestUtils, TestCase):
    def setUp(self):
        self.user = self.login()

    def test_menu_items(self):
        response = self.client.get(reverse("wagtailadmin_home"))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Miscellaneous")
        self.assertContains(response, "The Calendar")
        self.assertContains(response, "The Greetings")

    def test_calendar_index_view(self):
        url = reverse("calendar:index")
        response = self.client.get(url)
        now = timezone.now()
        self.assertEqual(url, "/admin/calendar/")
        self.assertContains(response, f"{now.year} calendar")

    def test_calendar_month_view(self):
        url = reverse("calendar:month")
        response = self.client.get(url)
        now = timezone.now()
        self.assertEqual(url, "/admin/calendar/month/")
        self.assertContains(response, f"{now.year}/{now.month} calendar")

    def test_greetings_view(self):
        self.user.first_name = "Gordon"
        self.user.last_name = "Freeman"
        self.user.save()
        url = reverse("greetings:index")
        response = self.client.get(url)
        self.assertEqual(url, "/admin/greetingz/")
        self.assertContains(response, "Greetings")
        self.assertContains(response, "Welcome to this greetings page, Gordon Freeman!")

    def test_method_injection(self):
        response = self.client.get(reverse("opera:index"))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Porgy and Bess")



import os

from django.core.exceptions import SuspiciousOperation
from django.db import transaction
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.encoding import force_str
from django.utils.functional import cached_property
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy, ngettext
from django.views.decorators.http import require_http_methods

from wagtail.admin import messages
from wagtail.admin.auth import PermissionPolicyChecker
from wagtail.admin.forms.search import SearchForm
from wagtail.admin.ui.tables import Column, StatusTagColumn, TitleColumn
from wagtail.admin.views import generic
from wagtail.admin.widgets.button import Button
from wagtail.contrib.frontend_cache.utils import PurgeBatch, purge_urls_from_cache
from wagtail.contrib.redirects.filters import RedirectsReportFilterSet
from wagtail.contrib.redirects.forms import (
    ConfirmImportForm,
    ConfirmImportManagementForm,
    ImportForm,
    RedirectForm,
)
from wagtail.contrib.redirects.models import Redirect
from wagtail.contrib.redirects.permissions import permission_policy
from wagtail.contrib.redirects.utils import (
    get_file_storage,
    get_format_cls_by_extension,
    get_import_formats,
    get_supported_extensions,
    write_to_file_storage,
)
from wagtail.log_actions import log
from wagtail.models import Site

permission_checker = PermissionPolicyChecker(permission_policy)


class RedirectTargetColumn(Column):
    cell_template_name = "wagtailredirects/redirect_target_cell.html"
    url_name = "wagtailadmin_pages:edit"

    def get_value(self, instance):
        if instance.redirect_page_id:
            return instance.redirect_page.get_admin_display_title()
        return instance.redirect_link

    def get_url(self, instance):
        if instance.redirect_page_id:
            return reverse(self.url_name, args=[instance.redirect_page_id])
        return None

    def get_cell_context_data(self, instance, parent_context):
        context = super().get_cell_context_data(instance, parent_context)
        context["url"] = self.get_url(instance)
        return context


class IndexView(generic.IndexView):
    template_name = "wagtailredirects/index.html"
    results_template_name = "wagtailredirects/index_results.html"
    permission_policy = permission_policy
    model = Redirect
    header_icon = "redirect"
    add_item_label = gettext_lazy("Add redirect")
    context_object_name = "redirects"
    index_url_name = "wagtailredirects:index"
    index_results_url_name = "wagtailredirects:index_results"
    add_url_name = "wagtailredirects:add"
    edit_url_name = "wagtailredirects:edit"
    delete_url_name = "wagtailredirects:delete"
    default_ordering = "old_path"
    paginate_by = 20
    page_title = gettext_lazy("Redirects")
    search_fields = ["old_path", "redirect_page__url_path", "redirect_link"]
    columns = [
        TitleColumn(
            "old_path",
            label=gettext_lazy("From"),
            url_name="wagtailredirects:edit",
            sort_key="old_path",
        ),
        Column(
            "site",
            label=gettext_lazy("Site"),
            width="25%",
            sort_key="site__site_name",
        ),
        RedirectTargetColumn(
            "redirect_page",
            label=gettext_lazy("To"),
            width="30%",
        ),
        StatusTagColumn(
            "is_permanent",
            accessor="get_is_permanent_display",
            label=gettext_lazy("Type"),
            width="10%",
            sort_key="is_permanent",
            primary=lambda r: r.is_permanent,
        ),
    ]
    filterset_class = RedirectsReportFilterSet
    list_export = [
        "old_path",
        "link",
        "get_is_permanent_display",
        "site",
    ]
    export_headings = {
        "old_path": _("From"),
        "site": _("Site"),
        "link": _("To"),
        "get_is_permanent_display": _("Type"),
    }

    def get_base_queryset(self):
        return super().get_base_queryset().select_related("redirect_page", "site")

    @cached_property
    def header_more_buttons(self) -> list[Button]:
        buttons = super().header_more_buttons.copy()
        buttons.append(
            Button(
                _("Import redirects"),
                url=reverse("wagtailredirects:start_import"),
                icon_name="doc-full-inverse",
                priority=50,
            )
        )
        return buttons


class EditView(generic.EditView):
    model = Redirect
    form_class = RedirectForm
    permission_policy = permission_policy
    template_name = "wagtailredirects/edit.html"
    index_url_name = "wagtailredirects:index"
    edit_url_name = "wagtailredirects:edit"
    delete_url_name = "wagtailredirects:delete"
    pk_url_kwarg = "redirect_id"
    error_message = gettext_lazy("The redirect could not be saved due to errors.")
    header_icon = "redirect"

    def get_success_message(self):
        return _("Redirect '%(redirect_title)s' updated.") % {
            "redirect_title": self.object.title
        }

    def save_instance(self):
        purger = PurgeBatch()

        root_paths = Site.get_site_root_paths()
        purger.add_urls(self.object.old_links(root_paths))

        instance = super().save_instance()

        purger.add_urls(self.object.old_links(root_paths))

        purger.purge()

        return instance


class DeleteView(generic.DeleteView):
    model = Redirect
    pk_url_kwarg = "redirect_id"
    permission_policy = permission_policy
    template_name = "wagtailredirects/confirm_delete.html"
    index_url_name = "wagtailredirects:index"
    delete_url_name = "wagtailredirects:delete"
    header_icon = "redirect"

    def delete_action(self):
        super().delete_action()
        purge_urls_from_cache(self.object.old_links())

    def get_success_message(self):
        return _("Redirect '%(redirect_title)s' deleted.") % {
            "redirect_title": self.object.title
        }


class CreateView(generic.CreateView):
    model = Redirect
    form_class = RedirectForm
    permission_policy = permission_policy
    template_name = "wagtailredirects/add.html"
    add_url_name = "wagtailredirects:add"
    index_url_name = "wagtailredirects:index"
    edit_url_name = "wagtailredirects:edit"
    error_message = gettext_lazy("The redirect could not be created due to errors.")
    header_icon = "redirect"

    def get_success_message(self, instance):
        return _("Redirect '%(redirect_title)s' added.") % {
            "redirect_title": instance.title
        }

    def save_instance(self):
        instance = super().save_instance()
        purge_urls_from_cache(instance.old_links())
        return instance


@permission_checker.require_any("add")
def start_import(request):
    supported_extensions = get_supported_extensions()
    from_encoding = "utf-8"

    query_string = request.GET.get("q", "")

    if request.POST or request.FILES:
        form_kwargs = {}
        form = ImportForm(
            supported_extensions,
            request.POST or None,
            request.FILES or None,
            **form_kwargs,
        )
    else:
        form = ImportForm(supported_extensions)

    if not request.FILES or not form.is_valid():
        return render(
            request,
            "wagtailredirects/choose_import_file.html",
            {
                "search_form": SearchForm(
                    data={"q": query_string} if query_string else None,
                    placeholder=_("Search redirects"),
                ),
                "form": form,
            },
        )

    import_file = form.cleaned_data["import_file"]

    _name, extension = os.path.splitext(import_file.name)
    extension = extension.lstrip(".")

    if extension not in supported_extensions:
        messages.error(
            request,
            _('File format of type "%(extension)s" is not supported')
            % {"extension": extension},
        )
        return redirect("wagtailredirects:start_import")

    import_format_cls = get_format_cls_by_extension(extension)
    input_format = import_format_cls()
    file_storage = write_to_file_storage(import_file, input_format)

    try:
        data = file_storage.read(input_format.get_read_mode())
        if not input_format.is_binary() and from_encoding:
            data = force_str(data, from_encoding)
        dataset = input_format.create_dataset(data)
    except UnicodeDecodeError as e:
        messages.error(
            request,
            _("Imported file has a wrong encoding: %(error_message)s")
            % {"error_message": e},
        )
        return redirect("wagtailredirects:start_import")
    except Exception as e:  # noqa: BLE001; pragma: no cover
        messages.error(
            request,
            _("%(error)s encountered while trying to read file: %(filename)s")
            % {"error": type(e).__name__, "filename": import_file.name},
        )
        return redirect("wagtailredirects:start_import")

    # This data is needed in the processing step, so it is stored in
    # hidden form fields as signed strings (signing happens in the form).
    initial = {
        "import_file_name": os.path.basename(file_storage.name),
        "input_format": get_import_formats().index(import_format_cls),
    }

    return render(
        request,
        "wagtailredirects/confirm_import.html",
        {
            "form": ConfirmImportForm(dataset.headers, initial=initial),
            "dataset": dataset,
        },
    )


@permission_checker.require_any("add")
@require_http_methods(["POST"])
def process_import(request):
    supported_extensions = get_supported_extensions()
    from_encoding = "utf-8"

    management_form = ConfirmImportManagementForm(request.POST)
    if not management_form.is_valid():
        # Unable to unsign the hidden form data, or the data is missing, that's suspicious.
        raise SuspiciousOperation(
            f"Invalid management form, data is missing or has been tampered with:\n"
            f"{management_form.errors.as_text()}"
        )

    input_format = get_import_formats()[
        int(management_form.cleaned_data["input_format"])
    ]()

    FileStorage = get_file_storage()
    file_storage = FileStorage(name=management_form.cleaned_data["import_file_name"])

    data = file_storage.read(input_format.get_read_mode())
    if not input_format.is_binary() and from_encoding:
        data = force_str(data, from_encoding)
    dataset = input_format.create_dataset(data)

    # Now check if the rest of the management form is valid
    form = ConfirmImportForm(
        dataset.headers,
        request.POST,
        request.FILES,
        initial=management_form.cleaned_data,
    )

    if not form.is_valid():
        return render(
            request,
            "wagtailredirects/confirm_import.html",
            {
                "form": form,
                "dataset": dataset,
            },
        )

    import_summary = create_redirects_from_dataset(
        dataset,
        {
            "from_index": int(form.cleaned_data["from_index"]),
            "to_index": int(form.cleaned_data["to_index"]),
            "permanent": form.cleaned_data["permanent"],
            "site": form.cleaned_data["site"],
        },
    )

    file_storage.remove()

    if import_summary["errors_count"] > 0:
        return render(
            request,
            "wagtailredirects/import_summary.html",
            {
                "form": ImportForm(supported_extensions),
                "import_summary": import_summary,
            },
        )

    total = import_summary["total"]
    messages.success(
        request,
        ngettext("Imported %(total)d redirect", "Imported %(total)d redirects", total)
        % {"total": total},
    )

    return redirect("wagtailredirects:index")


def create_redirects_from_dataset(dataset, config):
    errors = []
    successes = 0
    total = 0

    for row in dataset:
        total += 1

        from_link = row[config["from_index"]]
        to_link = row[config["to_index"]]

        data = {
            "old_path": from_link,
            "redirect_link": to_link,
            "is_permanent": config["permanent"],
        }

        if config["site"]:
            data["site"] = config["site"].pk

        form = RedirectForm(data)
        if not form.is_valid():
            error = to_readable_errors(form.errors.as_text())
            errors.append([from_link, to_link, error])
            continue

        with transaction.atomic():
            redirect = form.save()
            log(instance=redirect, action="wagtail.create")
        successes += 1

    return {
        "errors": errors,
        "errors_count": len(errors),
        "successes": successes,
        "total": total,
    }


def to_readable_errors(error):
    errors = error.split("\n")
    errors = errors[1::2]
    errors = [x.lstrip("* ") for x in errors]
    errors = ", ".join(errors)
    return errors



import hashlib
import os

from django.conf import STATICFILES_STORAGE_ALIAS, settings
from django.contrib.staticfiles.storage import HashedFilesMixin
from django.core.files.storage import storages
from django.templatetags.static import static

from wagtail import __version__

# Check whether we should add cache-busting '?v=...' parameters to static file URLs
try:
    # If a preference has been explicitly stated in the WAGTAILADMIN_STATIC_FILE_VERSION_STRINGS
    # setting, use that
    use_version_strings = settings.WAGTAILADMIN_STATIC_FILE_VERSION_STRINGS
except AttributeError:
    # If WAGTAILADMIN_STATIC_FILE_VERSION_STRINGS not specified, default to version strings
    # enabled, UNLESS we're using a storage backend with hashed filenames; in this case having
    # a query parameter is redundant, and in some configurations (e.g. Cloudflare with the
    # "No Query String" setting) it could break a previously-working cache setup

    if settings.DEBUG:
        # Hashed filenames are disabled in debug mode, so keep the querystring
        use_version_strings = True
    else:
        # see if we're using a storage backend using hashed filenames
        use_version_strings = not isinstance(
            storages[STATICFILES_STORAGE_ALIAS], HashedFilesMixin
        )


if use_version_strings:
    # INSTALLED_APPS is used as a unique value to distinguish Wagtail apps
    # and avoid exposing the Wagtail version directly
    VERSION_HASH = hashlib.sha1(
        "".join([__version__] + list(settings.INSTALLED_APPS)).encode(),
    ).hexdigest()[:8]
else:
    VERSION_HASH = None


if os.environ.get("WAGTAIL_FAIL_ON_VERSIONED_STATIC", "0") == "1":

    def versioned_static(path):
        raise Exception(
            "`versioned_static` was called during application startup. This is not valid "
            "as it will cause failures if collectstatic has not yet completed (e.g. during "
            "the collectstatic command itself). Ensure that any media definitions declared "
            "via `class Media` are converted to a `media` property."
        )
else:

    def versioned_static(path):
        """
        Wrapper for Django's static file finder to append a cache-busting query parameter
        that updates on each Wagtail version
        """
        # An absolute path is returned unchanged (either a full URL, or processed already)
        if path.startswith(("http://", "https://", "/")):
            return path

        base_url = static(path)

        # if URL already contains a querystring, don't add our own, to avoid interfering
        # with existing mechanisms
        if VERSION_HASH is None or "?" in base_url:
            return base_url
        else:
            return base_url + "?v=" + VERSION_HASH



from django.conf import settings
from django.contrib import admin

from wagtail.documents.models import Document

if (
    hasattr(settings, "WAGTAILDOCS_DOCUMENT_MODEL")
    and settings.WAGTAILDOCS_DOCUMENT_MODEL != "wagtaildocs.Document"
):
    # This installation provides its own custom document class;
    # to avoid confusion, we won't expose the unused wagtaildocs.Document class
    # in the admin.
    pass
else:
    admin.site.register(Document)



# Generated by Django 6.0.dev20250303103700 on 2025-03-21 07:17

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0049_promotionalpage"),
        ("wagtailcore", "0094_alter_page_locale"),
    ]

    operations = [
        migrations.AddField(
            model_name="headcountrelatedmodelusingpk",
            name="related_page",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="head_count_relations",
                to="wagtailcore.page",
            ),
        ),
    ]



from unittest import mock

from django.contrib.auth.models import Group, Permission
from django.test import TestCase
from django.urls import reverse

from wagtail.models import GroupPagePermission, Page
from wagtail.test.testapp.models import BusinessIndex, EventIndex, EventPage, SimplePage
from wagtail.test.utils import WagtailTestUtils
from wagtail.test.utils.template_tests import AdminTemplateTestUtils


class TestParentPageChooserView(AdminTemplateTestUtils, WagtailTestUtils, TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        super().setUp()
        self.user = self.login()
        self.view_url = reverse("event_pages:choose_parent")

    def test_get_page_parent_chooser(self):
        response = self.client.get(self.view_url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/pages/choose_parent.html")
        self.assertBreadcrumbsItemsRendered(
            [
                {"url": reverse("event_pages:index"), "label": "Event pages"},
                {"url": "", "label": "Choose parent", "sublabel": "Event page"},
            ],
            response.content,
        )

    def test_parent_chooser_redirect(self):
        parent_page = Page.objects.first()
        form_data = {
            "parent_page": parent_page.pk,
        }

        response = self.client.post(self.view_url, form_data)
        self.assertRedirects(
            response,
            reverse(
                "wagtailadmin_pages:add", args=("tests", "eventpage", parent_page.pk)
            ),
        )

        # Test another parent to make sure everything is working as intended
        another_parent = parent_page.get_first_child()
        form_data["parent_page"] = another_parent.pk

        response = self.client.post(self.view_url, form_data)
        self.assertRedirects(
            response,
            reverse(
                "wagtailadmin_pages:add", args=("tests", "eventpage", another_parent.pk)
            ),
        )

    def test_no_parent_selected(self):
        error_html = """<p class="error-message">This field is required.</p>"""

        response = self.client.post(self.view_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, error_html, html=True)

    def test_user_no_add_subpage_permission(self):
        parent_page = Page.objects.first()
        test_group = Group.objects.create(name="test_group")
        test_group.permissions.add(Permission.objects.get(codename="access_admin"))

        page_with_add_permission = Page(title="Page not to be selected")
        page_with_no_permission = Page(title="Page to be selected")
        parent_page.add_child(instance=page_with_add_permission)
        parent_page.add_child(instance=page_with_no_permission)

        GroupPagePermission.objects.create(
            group=test_group,
            page=page_with_add_permission,
            permission_type="add",
        )
        form_data = {
            "parent_page": page_with_no_permission.pk,
        }

        self.user.is_superuser = False
        self.user.groups.add(test_group)
        self.user.save()

        response = self.client.post(self.view_url, form_data)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            " You do not have permission to create a page under &quot;%s&quot;. "
            % page_with_no_permission.get_admin_display_title(),
        )

        form_data["parent_page"] = page_with_add_permission.pk
        response = self.client.post(self.view_url, form_data)
        self.assertRedirects(
            response,
            reverse(
                "wagtailadmin_pages:add",
                args=("tests", "eventpage", page_with_add_permission.pk),
            ),
        )

    def test_choosing_parent_with_unsupported_subpage_type(self):
        parent_page = Page.objects.first()
        page_with_limited_subtypes = BusinessIndex(title="EventPage unsupported")
        parent_page.add_child(instance=page_with_limited_subtypes)
        form_data = {
            "parent_page": page_with_limited_subtypes.pk,
        }

        response = self.client.post(self.view_url, form_data)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            "You cannot create a page of type &quot;Event page&quot; under &quot;%s&quot;."
            % page_with_limited_subtypes.get_admin_display_title(),
        )

    def test_skip_if_only_one_valid_parent(self):
        self.assertEqual(EventIndex.objects.count(), 1)

        with mock.patch.object(EventPage, "allowed_parent_page_models") as mock_method:
            mock_method.return_value = [EventIndex]
            self.assertEqual(EventPage.allowed_parent_page_models(), [EventIndex])
            response = self.client.get(self.view_url)

        self.assertRedirects(
            response,
            reverse(
                "wagtailadmin_pages:add",
                args=("tests", "eventpage", EventIndex.objects.first().pk),
            ),
        )

    def test_skip_if_user_only_has_permission_for_one_parent(self):
        test_group = Group.objects.create(name="test_group")
        test_group.permissions.add(Permission.objects.get(codename="access_admin"))

        self.user.is_superuser = False
        self.user.groups.add(test_group)
        self.user.save()

        # Only grant the user permission to add pages under one leaf page
        parent_page = SimplePage.objects.filter(numchild=0).first()
        GroupPagePermission.objects.create(
            group=test_group,
            page=parent_page,
            permission_type="add",
        )

        # Should redirect to the add view with the only valid parent page
        response = self.client.get(self.view_url)
        self.assertRedirects(
            response,
            reverse(
                "wagtailadmin_pages:add",
                args=("tests", "eventpage", parent_page.pk),
            ),
        )



from django.apps import AppConfig


class WagtailSnippetsTestsAppConfig(AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = "wagtail.test.streamfield_migrations"
    label = "streamfield_migration_tests"
    verbose_name = "Wagtail StreamField migration tests"



from unittest import mock

from django.contrib.auth.models import Permission
from django.http import HttpRequest, HttpResponse
from django.test import TestCase
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from wagtail.admin.views.pages.bulk_actions.page_bulk_action import PageBulkAction
from wagtail.models import Page
from wagtail.signals import page_unpublished
from wagtail.test.testapp.models import SimplePage
from wagtail.test.utils import WagtailTestUtils


class TestBulkUnpublish(WagtailTestUtils, TestCase):
    def setUp(self):
        # Create pages to unpublish
        self.root_page = Page.objects.get(id=2)
        self.child_pages = [
            SimplePage(
                title=f"Hello world!-{i}", slug=f"hello-world-{i}", content=f"hello-{i}"
            )
            for i in range(1, 5)
        ]
        # first three child pages will be unpublished
        self.pages_to_be_unpublished = self.child_pages[:3]
        self.pages_not_to_be_unpublished = self.child_pages[3:]
        for child_page in self.child_pages:
            self.root_page.add_child(instance=child_page)

        self.url = (
            reverse(
                "wagtail_bulk_action",
                args=(
                    "wagtailcore",
                    "page",
                    "unpublish",
                ),
            )
            + "?"
        )
        for child_page in self.pages_to_be_unpublished:
            self.url += f"&id={child_page.id}"
        self.redirect_url = reverse("wagtailadmin_explore", args=(self.root_page.id,))

        # Login
        self.user = self.login()

    def test_unpublish_view(self):
        """
        This tests that the unpublish view responds with an unpublish confirm page
        """
        # Request confirm unpublish page
        response = self.client.get(self.url)

        # Check that the user received an unpublish confirm page
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "wagtailadmin/pages/bulk_actions/confirm_bulk_unpublish.html"
        )

    def test_unpublish_view_invalid_page_id(self):
        """
        This tests that the unpublish view returns an error if the page id is invalid
        """
        # Request confirm unpublish page but with illegal page id
        response = self.client.get(
            reverse(
                "wagtail_bulk_action",
                args=(
                    "wagtailcore",
                    "page",
                    "unpublish",
                ),
            )
        )

        # Check that the user received a 404 response
        self.assertEqual(response.status_code, 404)

    def test_unpublish_view_bad_permissions(self):
        """
        This tests that the unpublish view doesn't allow users without unpublish permissions
        """
        # Remove privileges from user
        self.user.is_superuser = False
        self.user.user_permissions.add(
            Permission.objects.get(
                content_type__app_label="wagtailadmin", codename="access_admin"
            )
        )
        self.user.save()

        # Request confirm unpublish page
        response = self.client.get(self.url)

        # Check that the user received a 200 redirected response
        self.assertEqual(response.status_code, 200)

        html = response.content.decode()

        self.assertInHTML(
            "<p>You don't have permission to unpublish these pages</p>", html
        )

        for child_page in self.pages_to_be_unpublished:
            self.assertInHTML(f"<li>{child_page.title}</li>", html)

    def test_unpublish_view_post(self):
        """
        This posts to the unpublish view and checks that the page was unpublished
        """
        # Connect a mock signal handler to page_unpublished signal
        mock_handler = mock.MagicMock()
        page_unpublished.connect(mock_handler)

        try:
            # Post to the unpublish page
            response = self.client.post(self.url)

            # Should be redirected to explorer page
            self.assertEqual(response.status_code, 302)

            # Check that the child pages were unpublished
            for child_page in self.pages_to_be_unpublished:
                self.assertFalse(SimplePage.objects.get(id=child_page.id).live)

            # Check that the child pages not to be unpublished remain
            for child_page in self.pages_not_to_be_unpublished:
                self.assertTrue(SimplePage.objects.get(id=child_page.id).live)

            # Check that the page_unpublished signal was fired
            self.assertEqual(mock_handler.call_count, len(self.pages_to_be_unpublished))

            for i, child_page in enumerate(self.pages_to_be_unpublished):
                mock_call = mock_handler.mock_calls[i][2]
                self.assertEqual(mock_call["sender"], child_page.specific_class)
                self.assertEqual(mock_call["instance"], child_page)
                self.assertIsInstance(mock_call["instance"], child_page.specific_class)
        finally:
            page_unpublished.disconnect(mock_handler)

    def test_after_unpublish_page(self):
        def hook_func(request, action_type, pages, action_class_instance):
            self.assertEqual(action_type, "unpublish")
            self.assertIsInstance(request, HttpRequest)
            self.assertIsInstance(action_class_instance, PageBulkAction)
            for i, page in enumerate(pages):
                self.assertEqual(page.id, self.pages_to_be_unpublished[i].id)

            return HttpResponse("Overridden!")

        with self.register_hook("after_bulk_action", hook_func):
            response = self.client.post(self.url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"Overridden!")

        for child_page in self.pages_to_be_unpublished:
            child_page.refresh_from_db()
            self.assertEqual(child_page.status_string, _("draft"))

    def test_before_unpublish_page(self):
        def hook_func(request, action_type, pages, action_class_instance):
            self.assertEqual(action_type, "unpublish")
            self.assertIsInstance(request, HttpRequest)
            self.assertIsInstance(action_class_instance, PageBulkAction)
            for i, page in enumerate(pages):
                self.assertEqual(page.id, self.pages_to_be_unpublished[i].id)

            return HttpResponse("Overridden!")

        with self.register_hook("before_bulk_action", hook_func):
            response = self.client.post(self.url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"Overridden!")

    def test_unpublish_descendants_view(self):
        """
        This tests that the unpublish view responds with an unpublish confirm page that does not contain the form field 'include_descendants'
        """
        # Get unpublish page for page with no descendants
        response = self.client.get(self.url)

        # Check that the user received an unpublish confirm page
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "wagtailadmin/pages/bulk_actions/confirm_bulk_unpublish.html"
        )
        # Check the form does not contain the checkbox field include_descendants
        self.assertContains(
            response,
            'name="include_descendants"',
            count=0,
        )


class TestBulkUnpublishIncludingDescendants(WagtailTestUtils, TestCase):
    def setUp(self):
        # Find root page
        self.root_page = Page.objects.get(id=2)

        self.child_pages = [
            SimplePage(
                title=f"Hello world!-{i}", slug=f"hello-world-{i}", content=f"hello-{i}"
            )
            for i in range(1, 5)
        ]
        # first three child pages will be unpublished
        self.pages_to_be_unpublished = self.child_pages[:3]
        self.pages_not_to_be_unpublished = self.child_pages[3:]
        for child_page in self.child_pages:
            self.root_page.add_child(instance=child_page)

        # map of the form { page: [child_pages] } to be added
        self.grandchildren_pages = {
            self.pages_to_be_unpublished[0]: [
                SimplePage(
                    title="Hello world!-a", slug="hello-world-a", content="hello-a"
                )
            ],
            self.pages_to_be_unpublished[1]: [
                SimplePage(
                    title="Hello world!-b", slug="hello-world-b", content="hello-b"
                ),
                SimplePage(
                    title="Hello world!-c", slug="hello-world-c", content="hello-c"
                ),
            ],
        }

        for child_page, grandchild_pages in self.grandchildren_pages.items():
            for grandchild_page in grandchild_pages:
                child_page.add_child(instance=grandchild_page)

        self.url = (
            reverse(
                "wagtail_bulk_action",
                args=(
                    "wagtailcore",
                    "page",
                    "unpublish",
                ),
            )
            + "?"
        )
        for child_page in self.pages_to_be_unpublished:
            self.url += f"&id={child_page.id}"
        self.redirect_url = reverse("wagtailadmin_explore", args=(self.root_page.id,))

        self.user = self.login()

    def test_unpublish_descendants_view(self):
        """
        This tests that the unpublish view responds with an unpublish confirm page that contains the form field 'include_descendants'
        """
        # Get unpublish page
        response = self.client.get(self.url)

        # Check that the user received an unpublish confirm page
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "wagtailadmin/pages/bulk_actions/confirm_bulk_unpublish.html"
        )
        # Check the form contains the checkbox field include_descendants
        self.assertContains(
            response,
            'name="include_descendants"',
        )

    def test_unpublish_include_children_view_post(self):
        """
        This posts to the unpublish view and checks that the page and its descendants were unpublished
        """
        # Post to the unpublish page
        response = self.client.post(self.url, {"include_descendants": "on"})

        # Should be redirected to explorer page
        self.assertEqual(response.status_code, 302)

        # Check that the child pages were unpublished
        for child_page in self.pages_to_be_unpublished:
            self.assertFalse(SimplePage.objects.get(id=child_page.id).live)

        # Check that the child pages not to be unpublished remain
        for child_page in self.pages_not_to_be_unpublished:
            self.assertTrue(SimplePage.objects.get(id=child_page.id).live)

        for grandchild_pages in self.grandchildren_pages.values():
            for grandchild_page in grandchild_pages:
                self.assertFalse(SimplePage.objects.get(id=grandchild_page.id).live)

    def test_unpublish_not_include_children_view_post(self):
        """
        This posts to the unpublish view and checks that the page was unpublished but its descendants were not
        """
        # Post to the unpublish page
        response = self.client.post(self.url, {})

        # Should be redirected to explorer page
        self.assertEqual(response.status_code, 302)

        # Check that the child pages were unpublished
        for child_page in self.pages_to_be_unpublished:
            self.assertFalse(SimplePage.objects.get(id=child_page.id).live)

        # Check that the descendant pages were not unpublished
        for grandchild_pages in self.grandchildren_pages.values():
            for grandchild_page in grandchild_pages:
                self.assertTrue(SimplePage.objects.get(id=grandchild_page.id).live)
